<?php
/**
 * This file provides different helper functions intended to be
 * used through ajax calls.
 */

require_once('include/config.inc.php');
require_once('classes/object_class.php');

if (isset($DEBUG) && $DEBUG) {
	require_once('vendor/FirePHPCore/fb.php');
	ob_start();
}

$action = false;
$data = false;
$retval = 0;
$obj_type = false;
$obj_id = false;

if (isset($_REQUEST['action'])) {
	$action = trim(addslashes($_REQUEST['action']));
}

/**
 * Checks if the current user will be given by-contact-access access to an
 * object with the specified contacts and contact groups.
 *
 * Always returns true if the current right set is not by-contact-access.
 * Returns true if the user is listed in the contacts, or member of the listed
 * contact groups. Returns false otherwise.
 */
function is_user_contact($obj_type, $contacts, $contact_groups, $template) {
	$access = get_nacoma_access($obj_type);
	if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
		return true;

	$username = get_user();
	$user = new contact_class($username);
	if ($template !== false) {
		$class = $obj_type.'_template_class';
		$template = new $class($template);
	}

	if (in_array($user->id, $contacts))
		return true;
	if (@in_array($user->id, $template->obj['contacts']))
		return true;
	if (isset($user->obj['contactgroups'])) {
		foreach($contact_groups as $group)
			if (in_array($group, $user->obj['contactgroups']))
				return true;
		if (@is_array($template->obj['contact_groups']))
			foreach ($template->obj['contact_groups'] as $group)
				if (in_array($group, $user->obj['contactgroups']))
					return true;
	}
	return false;
}

switch ($action) {
	case 'object_data':
		if (isset($_REQUEST['obj_type'])) {
			$obj_type = trim($_REQUEST['obj_type']);
			$obj_type = str_replace('new_', '', $obj_type);
		}

		$obj_type = str_replace('new_', '', $obj_type);
		if (isset($_REQUEST['obj_id'])) {
			$obj_id = $_REQUEST['obj_id'];
		}
		$valid_obj_types = array('host', 'service', 'contact', 'host_template', 'service_template', 'contact_template');

		if (!$obj_type || !in_array($obj_type, $valid_obj_types)) {
			header("HTTP/1.0 400 Bad request");
			echo "Can not resolve object type ($obj_type) or object id ($obj_id)";
			ob_end_flush();
			exit;
		}

		$class_name = $obj_type . '_class';
		$obj_id = $obj_id == 0 ? 'new' : $obj_id;
		$obj = new $class_name($obj_id);
		if ($obj_id  && $obj_id) {
			$obj->expand_object();
			$data = $obj->denormalize();
		} else {
			$data = $obj->get_default_object();
		}
		break;

	case 'hosts_with_svc':
		require_once(dirname(__FILE__) . '/include/data.inc.php');
		if (isset($_REQUEST['service']) && !empty($_REQUEST['service'])) {
			$svc = trim($_REQUEST['service']);
			$sql = "SELECT DISTINCT host.host_name FROM host INNER JOIN service ON service.host_name = host.id WHERE service.service_description = '".$svc."'";
			$res = pdo_exec_query($sql);
			if ($res != false && count($res) > 0) {
				while ($row = $res->fetch(PDO::FETCH_ASSOC)) {
					$data[] = $row['host_name'];
				}
			}
		}
		break;

	/**
	 * Takes hosts, sends back services allowed, i.e. those
	 * who have graph data available
	 */
	case 'pnp_page_services_for_hosts':
		if(!isset($_GET['hosts']) || !$_GET['hosts']) {
			// do not run expensive runs if we don't want to
			$data = array();
			break;
		}
		$available_graphs = pnp_page_class::get_available_graphs();
		$hosts_to_examine = $_GET['hosts'];

		// @todo accept sent in services? benchmark
		$available_services = array();
		foreach($available_graphs as $host => $graph) {
			if(in_array($host, $hosts_to_examine)) {
				$services = array_keys($graph);
				$available_services = array_merge($available_services, $services);
			}
		}
		$data = $available_services;
		break;

	case 'extend_lock':
		if (!isset($_REQUEST['obj_type']) || empty($_REQUEST['obj_type']))
			break;
		if (!isset($_REQUEST['obj_id']) || empty($_REQUEST['obj_id']))
			break;
		require_once(dirname(__FILE__).'/include/objlock.inc.php');
		$retval = extend_lock($_REQUEST['obj_type'], $_REQUEST['obj_id']);
		$retval = $retval === true ? 1 : 0;
		break;

	case 'is_user_contact':
		require_once(dirname(__FILE__) . '/include/nacoma_access.inc.php');
		if (!isset($_REQUEST['objs']) || empty($_REQUEST['objs']))
			break;
		foreach ($_REQUEST['objs'] as $obj) {
			if (!isset($obj['obj_type']) || empty($obj['obj_type']))
				continue;
			if (isset($obj['contacts']))
				$contacts = explode(';', $obj['contacts']);
			else
				$contacts = array();
			if (isset($obj['contact_groups']))
				$cgroups = explode(';', $obj['contact_groups']);
			else
				$cgroups = array();
			$template = isset($obj['template']) ? $obj['template'] : false;
			$retval = is_user_contact($obj['obj_type'], $contacts, $cgroups, $template) ? 1 : 0;
			if ($retval == 0)
				break 2;
		}
		break;
	case 'services_for_hostgroup':
		if (!isset($_REQUEST['obj_id']))
			break;
		$hg = new hostgroup_class($_REQUEST['obj_id']);
		$slaves = $hg->find_slave_objects('service');
		$data = array();
		foreach($slaves as $slave) {
			$svc = new service_class($slave);
			$data[] = $svc->get_object_name();
		}
		break;
	case 'merlin_nodes':
		require_once('include/probe.inc.php');
		require_once('include/system.inc.php');
		$data = get_merlin_nodes();
		break;
}

if (!empty($data)) {
	if (!function_exists('json_encode')) {
		require_once($_SERVER['DOCUMENT_ROOT']. "/op5common/lib/json_class.php");
		$json = new JSON();
		$json_data = $json->encode($data);
		unset($json);
	} else {
		$json_data = json_encode($data);
	}
	echo $json_data;
} else {
	// single value returned so no json required
	echo $retval;
}

if (isset($DEBUG) && $DEBUG)
	ob_end_flush();
