<?php

require_once(dirname(__FILE__).'/../object_class.php');
require_once('base_controller.php');

/**
 * Controller for configuring user access
 */
class access_controller extends base_controller {
	var $name = 'Local Users';

	/**
	 * @return array [username] => userobj
	 */
	private function get_users()
	{
		$users = array();
		foreach (user_class::get_users() as $user)
			$users[$user->get_object_name()] = $user;
		ksort($users);
		return $users;
	}

	/**
	 * Verify user permissions
	 */
	function __construct($input) {
		$access = get_nacoma_access('access_rights');
		if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access))
			deny_access("You need full 'access_rights' permissions to modify permissions.");

		$this->input = $input;

	}

	/**
	 * Print user access list
	 */
	function index() {
		$objects = $this->get_users();
		$user = new user_class('new');
		$objects['new'] = $user;
		$description = '<strong>Manage Local Users</strong></p>
<p>This is where you configure which users are able to log into monitor using modules that require user configuration.</p>
<div class="note"><p><strong>Please note:</strong> If you change your password or delete yourself you will be logged out.</p></div>';
		$input = $this->input;
		require_once(dirname(__FILE__).'/../../templates/many-per-page.php');
	}

	/**
	 * Delete user
	 */
	function delete() {
		user_class::get_users();
		$deluser = $_REQUEST['delete'];
		echo "<p><b>Removing user '".gui_class::escape_html($deluser)."'</b></p>\n";

		$user = new user_class($deluser);
		$user->delete_object();

		// @todo this really should be a redirect...
		$this->index();
	}

	/**
	 * Save user changes
	 */
	function Submit() {
		global $Config, $page;
		user_class::get_users();
		$users = $_REQUEST['user'];
		$errors = array();

		foreach ($users as $userid => $userobj) {
			if('new' == $userid && !$userobj['username'] && !$userobj['password'][1] && !$userobj['password'][2]) {
				// skip validating the 'new' user if it's empty
				continue;
			}
			$user = new user_class($userid);
			$user->lazy_write = true;
			$user->obj = $userobj;
			if(!$user->save_object()) {
				$errors[$userid] = $user->validation_error_list['password'];
			}
		}

		if($errors) {
			$page->print_validation_error_table(new user_class(), $errors);
		} else {
			try {
				user_class::write_users();
				print "<p><img src=\"images/icons/shield-ok_16x16.png\" alt=\"\" style=\"margin-bottom: -3px\" /> <strong>Successfully saved users</strong></p>";
			} catch (RuntimeException $ex) {
				$page->print_validation_error_table(new user_class(), array('saving' => "Error saving users: ".$ex->toString().". Make sure the HTTP user can write your user configuration."));
			}
		}

		// @todo this really should be a redirect...
		$this->index();
	}
}
