<?php
require_once('op5/license/op5lic.php');

/**
 * A whole class dedicated only to that tiny thing at the top of the page. Wow!
 *
 * Sort of silly as a whole class, but we need to be able to add warning
 * messages during rendering, so it's better to have all the messages in one
 * place so they don't go out of sync.
 */
class top_bar_class {
	private static $unsaved_changes = 'You have unsaved configuration changes. Please press SAVE to verify and activate.';

	/**
	 * Render the top bar
	 */
	public static function render() {
		$configured = get_num_hosts_configured();
		$lic = new op5lic();

		$to_show = array();
		$ret = false;

		if (!$lic->query_threshold('monitor', $configured, 0.95, $message)) {
			$message = implode('<br/>', $message);
			$to_show[] = '<span id="too_many">'.$message.'</span>';
		}

		if ((bool) Changelog::get_changes_count())
			$to_show[] = '<span id="unsaved">'.self::$unsaved_changes.'</span>';

		if (empty($to_show))
			return '<div class="config-warning"></div>';

		$ret .= '<div class="config-warning">';
		if (count($to_show) === 1)
			$ret .= '<p style="padding-top: 6px;">'.$to_show[0].'</p>';
		else
			$ret .= '<p>'.implode($to_show).'</p>';
		$ret .= '</div>';

		return $ret;
	}

	/**
	 * Add warning to the top-bar that there are changed but
	 * unsaved objects in the database
	 */
	public static function add_unsaved_warning() {
		if ((bool) Changelog::get_changes_count()) {
			?>
			<script type="text/javascript">
				$('.config-warning #unsaved').remove();
				if ($('.config-warning p').length == 0)
					$('.config-warning').append('<p></p>');

				$('.config-warning p').append('<span id="unsaved"><?php echo self::$unsaved_changes; ?></span>');
				if ($('.config-warning span').length > 1)
					$('.config-warning p').attr('style', '');
				else if ($('.config-warning span').length == 1)
					$('.config-warning p').attr('style', 'padding-top: 6px');
			</script>
			<?php
		}
	}

	/**
	 * Remove the unsaved changes warning from the top bar
	 */
	public static function remove_unsaved_warning() {
		?>
		<script type="text/javascript">
			$('.config-warning #unsaved').remove();
			if ($('.config-warning span').length == 0)
				$('.config-warning p').attr('style', 'padding: 0');
		</script>
		<?php
	}

	/**
	 * Adds warning to the top-bar with the supplied message.
	 */
	public static function add_warning($msg) {
		?>
		<script type="text/javascript">
			$('.config-warning #add_warning').remove();
			if ($('.config-warning p').length == 0) {
				$('.config-warning').append('<p></p>');
			}

			$('.config-warning p').append('<span id="add_warning"><?php echo $msg; ?></span>');
			if ($('.config-warning span').length > 1)
				$('.config-warning p').attr('style', '');
			else if ($('.config-warning span').length == 1)
				$('.config-warning p').attr('style', 'padding-top: 6px');
		</script>
		<?php
	}
}
