<?php
require_once('classes/object_class.php');
require_once('include/db_config.inc.php');
require_once('include/nacoma_access.inc.php');

require_once('op5/queryhandler.php');

function merge_obj() {
	$result = array();
	foreach( func_get_args() as $arr ) {
		if( is_array($arr) ) {
			foreach( $arr as $k => $v ) {
				if( !isset($result[$k]) ) {
					$result[$k] = $v;
				} else if( is_array($result[$k]) ) {
					$result[$k] = merge_obj($result[$k], $v);
				} else if( $v !== "" ) {
					/* This feels kind of strange... but don't overwrite hostgroups */
					$result[$k] = $v;
				}
			}
		}
	}
	return $result;
}
function apply_custom_vars($obj, $q) {
	if( isset($q['custom_vars']) ) {
		$qcv = $q['custom_vars'];
	} else {
		return;
	}

	if( isset($obj->type) && isset($qcv[$obj->type]) ) {
		$qcv = $qcv[$obj->type];
	} else {
		return;
	}

	if( isset($obj->id) && isset($qcv[$obj->id]) ) {
		$qcv = $qcv[$obj->id];
	} else if( empty($obj->id) && isset($qcv['new']) ) {
		$qcv = $qcv['new'];
	} else {
		return;
	}

	$custom_vars = array();
	foreach($qcv as $pair) {
		$custom_vars[$pair['key']] = $pair['value'];
	}
	$obj->custom_vars = $custom_vars;
}

function strip_empty($arr) {
	$result = array();
	foreach( $arr as $k => $v ) {
		if( is_array($v) ) {
			$result[$k] = strip_empty($v);
		} else if( $v !== "" ) {
			$result[$k] = $v;
		}
	}
	return $result;
}

function print_escaped($text, $do_escape = true) {
	if( $do_escape )
		print htmlentities($text);
	else
		print $text;
}


function get_escape_html_tags() {
	global $Config;
	$lines = file($Config['cgi']);
	if( $lines === false )
		return null;
	foreach($lines as $line) {
		if( preg_match(':[\s]*escape_html_tags[\s]*=[\s]*([^\s]+)[\s]*:', $line, $parts) ) {
			return 0 != $parts[1];
		}
	}
	return true; /* Default: escape */
}

header('Cache-Control: no-cache');

$q = $_GET;

$obj = array();
$cmd_node = false;
if (!empty($q['cmdTestNode']))
	$cmd_node = $q['cmdTestNode'];

$do_escape = get_escape_html_tags();

$host = false;
$service = false;
$command = false;
$runcmd = false;
$objtype = false;

if (isset($q['obj_type']) && $q['obj_type'] == 'service') {
	$access = get_nacoma_access('test_this_service');
	if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
		exit('You are not allowed to test this service.');
	}
	$service = new service_class($q['tmp_obj_id']);
	$service->obj = strip_empty(merge_obj($service->obj, $q[$q['page_selection']][$q['tmp_obj_id']]));
	apply_custom_vars($service, $q);
	$service->template_expand_object();

	$host_id = false;
	$hostgroup_id = false;
	if( isset($q['master_obj_id']) ) {
		$master_id = intval($q['master_obj_id']);
		if( isset($q['master_obj_type']) ) {
			switch( $q['master_obj_type'] ) {
				case 'host':
					$host_id = $master_id;
					break;
				case 'hostgroup':
					$hostgroup_id = $master_id;
					break;
			}
		}
	} else {
		$host_id = $service->master_obj_id;
	}

	if( $hostgroup_id !== false ) {
		$hostgroup = new hostgroup_class($hostgroup_id);
		if( isset($hostgroup->obj)
				&& isset($hostgroup->obj['members'])
				&& count($hostgroup->obj['members']) > 0) {
			$host_id = $hostgroup->obj['members'][0];
		}
	}

	if( $host_id !== false ) {
		$host = new host_class($host_id);
		apply_custom_vars($host, $q);
		$host->template_expand_object();
	}

	$command_id = $service->obj['check_command'];
	$command = new command_class($command_id);
	$objtype = 'service';
}
if (isset($q['obj_type']) && $q['obj_type'] == 'host') {
	$access = get_nacoma_access('test_this_host');
	if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
		exit('You are not allowed to test this host.');
	}
	$host = new host_class(intval($q['tmp_obj_id']));
	$host->obj = strip_empty(merge_obj($host->obj, $q[$q['page_selection']][$q['tmp_obj_id']]));
	apply_custom_vars($host, $q);
	$host->template_expand_object();

	$command_id = $host->obj['check_command'];
	$command = new command_class($command_id);
	$objtype = 'host';
}
if (isset($_GET['cmd_str'])) {
	$access = get_nacoma_access('test_this_command');
	if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
		exit('You are not allowed to test this command.');
	}
	$command = new command_class();
	$command->obj['command_line'] = $_GET['cmd_str'];
	$command->obj['command_name'] = 'test_command';
	$objtype = 'command';
}

$cmdrun = new cmd_test_class();

if( $command !== false ) $cmdrun->attach_object( $command );
if( $service !== false ) $cmdrun->attach_object( $service );
if( $host    !== false ) $cmdrun->attach_object( $host );

try {
	print_escaped($cmdrun->cmdline($objtype)."\n", true);
	print_escaped($cmdrun->runcmd($objtype, $cmd_node), $do_escape);
} catch( op5queryhandler_Exception $e ) {
	if( $DEBUG ) {
		print $e;
	} else {
		print $e->getMessage();
	}
}
