<?php
/*
 * Copyright(C) 2003-2005, 2007-2008 op5 AB
 * All rights reserved.
 */

require_once('classes/common_gui.php');
require_once('include/object_properties.inc.php');
require_once('include/data.inc.php');
require_once('include/changelog.inc.php');
require_once('include/export.inc.php');
require_once('include/check_mk.inc.php');
require_once('include/objlock.inc.php');

require_once('op5/license/op5lic.php');

if (!isset($console_access)) {
	$page->html_start();
}

die_if_locked_hard();

$access = get_nacoma_access('export');
if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access))
	deny_access("You're not authorized to save config changes");

$page->import_if_changes();

$lic = new op5lic();
/* Disable save if license is exceeded */
$num_configured = get_num_hosts_configured();
if (!$lic->query(array('monitor' => $num_configured, 'time_expire' => false), $message)) {
	echo "<h2>The request to save the configuration was rejected</h2>\n";
	echo '<p>'.implode('<br/>', $message).'  <a href="/monitor/index.php/license" target="_parent">Click here for more information on how to upgrade your license.</a></p>';
	$page->html_end();
}

if (!isset($_REQUEST['action']) || $_REQUEST['action'] === 'full')
	$changes = new Changelog(null, null, true);
else if (isset($_REQUEST['action']) && $_REQUEST['action'] === 'force')
	$changes = true;
else
	$changes = new Changelog(null, null, false);

$rights = get_rights();

/* Press the button to actually export, or we die here */
if(!isset($console_access) && !isset($_REQUEST['action'])) {
	if (!count($changes)) {
		if (PHP_SAPI !== 'cli' && !defined('TEST_RUN')) {
			echo '<img src="images/icons/shield-ok.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
			echo '<h2>Nothing to save</h2></p>';
			if (isset($rights['configuration_all'])) {
				$page->form_start();
				$page->hidden_var('action', 'force');
				$page->submit_button('Rewrite configuration anyway', 'action_dummy');
				$page->form_end();
				echo '<script>$(function() {
					$("#foo_form").submit(function() {
						$(this)
							.find("input[type=submit]")
								.val("Rewriting configuration, do not leave this page. You will be directed when the save is completed.")
								.attr("disabled", "disabled");
					});
				});</script>';
			}
		}
		$page->html_end();
	}
	echo '<img src="images/icons/shield-unknown.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
	echo "<h2>You are about to save your configuration.</h2>\n";
	echo "<p>This will overwrite your current configuration, so make sure\n";
	echo "you've done everything right.<br /><br />\n";
	$users = $changes->get_other_users();
	if (count($users)) {
		echo '<div></div>'; // Move the img down to h2's top position
		echo '<img src="images/icons/shield-warning_24x24.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
		print "<h2>Other users' changes will be included in the save as well</h2>";
		print "<p>Be sure to review those changes below before saving.</p>";
	}
	print "<h3>Changes</h3>";
	print '<p class="changesummary">'.$changes->get_summary().' <a href="#">More info</a>.</p>';
	print $changes->get_html_list();

	echo "<p><b>Do you want to save your new configuration?</b></p>";
	echo "<table border='0'>";
	echo "<tr><td align='right'>\n";
	echo "<form method='post' action='$_SERVER[PHP_SELF]'>\n";
	echo "<input type='hidden' name='action' value='local'>\n";
	echo "<input type='submit' name='x' value='Save objects I have changed'>\n";
	echo "</form></td>";
	if (isset($rights['configuration_all'])) {
		echo "<td width='10'></td><td align='right'>\n";
		echo "<form method='post' action='$_SERVER[PHP_SELF]'>\n";
		echo "<input type='hidden' name='action' value='full'>\n";
		echo "<input type='submit' name='x' value='Save everything'>\n";
		echo "</form></td>";
	}
	echo "</tr></table>\n";
	echo "</div>\n";
	$page->html_end();
}

if (($err = do_export($changes)) !== true) {
	switch ($err['type']) {
	 case 'save in progress':
		echo '<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
		echo "<h2>A save is already in progress</h2>\n";
		echo "<p>You must wait for that to complete, before you can save more changes.</p>\n";
		break;
	 case 'nothing to do':
		echo '<img src="images/icons/shield-ok.png" alt="" style="display: inline; float: left; margin-right: 10px;" /><h2>Nothing to save</h2>';
		break;
	 case 'file write error':
		$files = $err['files'];
		$grp_info = posix_getgrgid(posix_getegid());
		$grp_name = $grp_info['name'];
		echo '<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
		echo "<h2>An irrecoverable error occurred</h2>\n";
		echo "<p><b>The following files are not writable by the webserver user:</b><br />\n";
		foreach ($files as $f)
			echo "&nbsp; &nbsp; $f<br />\n";

		echo "<br />\n";
		echo "<b>To correct this problem, log in as root to your system and run\n";
		echo "the following commands:</b><pre>\n";
		echo "  chown monitor:$grp_name " . join(' ', $files) . "\n";
		echo "  chmod 664 " . join(' ', $files) . "\n";
		echo '</p>';
		break;
	 case 'FILE':
		echo '<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
		echo "<h2>FILE not set bug encountered.</h2>\n";
		echo "<p>To avoid ruining your configuration, I have aborted the save</p>";
		echo "<p>Please contact op5 as soon as possible. It is important that you ";
		echo "include your configuration, as well as the changes you were saving.</p>";
		break;
	 case 'preflight check failed':
		echo '<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
		echo "<h2>Preflight configuration check failed!</h2>\n";
		echo "<p>The existing, on-disk, configuration contains errors.</p>\n";
		echo "<p>Restarting not attempted. Your changes have not been saved.<br />" .
			"If you think this to be a bug, please contact $SUPPORT.<br />" .
			"The output of the verification command below will be very useful.</p>";
		echo "<pre>".$err['output']."</pre>";
		break;
	 default:
		echo '<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
		echo '<h2>Unknown Error</h2>';
		break;
	}
	$page->html_end();
}

echo '<img src="images/icons/shield-ok.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
echo "<h2>Preflight configuration check turned out ok.</h2><p>\n";

# Try to reload monitor if it's running and its command-pipe is writable
$result = export_reload_monitor();

if ($result !== 0) {
	echo '<img src="images/icons/shield-critical.png" alt="" style="display: inline; float: left; margin-right: 10px;" />';
	if ($result === 1) {
		echo "<h2>Monitor doesn't appear to be running.</h2>\n";
		echo "<p>This could indicate a more serious error, so restarting not attempted.\n</p>\n";
	}
	elseif ($result === 2) {
		echo "<h2>Failed to open monitor command pipe for writing!</h2>\n";
		echo "<p>This can happen when Monitor isn't running.</p>\n";
	}
	echo "<p>To fix this problem, you can log in to your op5 Server as" .
		" root via Secure Shell (SSH) and issue the following commands;<br />\n";
	echo "<b><pre>\n";
	echo "   service monitor stop\n";
	echo "   service monitor slay\n";
	echo "   service monitor start\n";
	echo "</pre></b>\n";
	echo "This will also force Monitor to load the new configuration, so you won't have to click 'save' again.</p>\n";
}
else {
	echo "Monitor has successfully loaded the new configuration.<br />\n";
	check_mk_config_activate();
}

echo top_bar_class::remove_unsaved_warning();
$page->html_end();
