<?php

$check_mk_base_conf_dir = "/etc/check_mk/conf.dxx";

/*
 * this function writes out the configuration
 * for check_mk
 * it generates the preflight file:
 * /etc/check_mk/conf.d/<hostname>.preflight
 */
function check_mk_config_create($host_id)
{
    global $check_mk_base_conf_dir;
    global $Object_Index_Table;

    if(!isset($hosts[$host_id])) { return; }
    $hostname = $hosts[$host_id];

    $safe_file_name = preg_replace("/[^a-zA-Z0-9\-_]/", '_', $hostname);
    $file = $check_mk_base_conf_dir."/".$safe_file_name.".preflight";

    if(file_exists($file)) {
        unlink($file);
    }

    // get id of check_mk_passive
    $commands   = array_flip(get_object_index_table('command'));
    $command_id = false;
    if(isset($commands['check_mk_passive'])) {
        $command_id = $commands['check_mk_passive'];
    }
    else {
        // did not find a check_mk_passive command, should not happen
        return;
    }

    $check_mk_config = "";
    foreach($services as $id => $service_obj) {
        if($service_obj['host_name'] != $host_id) {
            continue;
        }
        if($service_obj['check_command'] != $command_id) {
            continue;
        }
        list($check_base, $check_args, $check_thresholds)
            = explode("!", $service_obj['check_command_args']);
        if($check_args != "None") {
            $check_args = "'".$check_args."'";
        }
        $check_mk_config .= sprintf("checks += [('%s', '%s', %s, %s )] \n", $hostname, $check_base, $check_args, $check_thresholds);
    }

    if(!empty($check_mk_config)) {
        $check_mk_config = "# this file is automatically generated by nacoma, do not edit.\n# changes will be overwritten\n\n".$check_mk_config;
        $fp = fopen($file, 'w+');
        fwrite($fp, $check_mk_config) or die("cannot write to file: ".$file);
        fwrite($fp, "\n");
        fclose($fp);
    }
}


/*
 * this function activates the previously written config
 * basically it just move the hostname.preflight to hostname.mk
 */
function check_mk_config_activate()
{
    global $check_mk_base_conf_dir;
	if(!file_exists($check_mk_base_conf_dir) || !is_dir($check_mk_base_conf_dir)){
		return;
	}
    if($handle = opendir($check_mk_base_conf_dir)) {
        while(false !== ($file = readdir($handle))) {
            if(preg_match("/\.preflight$/", $file)) {
                $new_file = preg_replace("/\.preflight$/", ".mk", $file);
                rename($check_mk_base_conf_dir."/".$file, $check_mk_base_conf_dir."/".$new_file);
            }
        }
        closedir($handle);
    }
    check_mk_config_cleanup();
}


/*
 * this function cleans preflight files
 */
function check_mk_config_cleanup()
{
    global $check_mk_base_conf_dir;
	if(!file_exists($check_mk_base_conf_dir) || !is_dir($check_mk_base_conf_dir)){
		return;
	}
    if($handle = opendir($check_mk_base_conf_dir)) {
        while(false !== ($file = readdir($handle))) {
            if(preg_match("/\.preflight$/", $file)) {
                unlink($check_mk_base_conf_dir."/".$file);
            }
        }
        closedir($handle);
    }
}

?>
