<?php
/*
 * Copyright (C) 2004, 2005, 2007, 2011 op5 AB
 * All rights reserved.
 *
 */

require_once('db_wrap.inc.php');
require_once('config.inc.php');
require_once('object_properties.inc.php');
require_once('webconfig.inc.php');
require_once('nacoma_access.inc.php');
require_once('changelog.inc.php');
require_once('op5/log.php');

/****************************************************************************/
/************************** OBJECT FUNCTIONS ********************************/
/****************************************************************************/

function get_super_users() {
	$opers = "";
	$buf = read_file($Config['cgi']);
	$lines = explode("\n", $buf);
	foreach($lines as $line) {
		$line = trim($line);
		if($line[0] == '#') continue;
		if(strstr($line, 'authorized_for_system_commands')) {
			$part = preg_split("/ *= */", $line, 2);
			$users = preg_split("/ *, */", $part[1]);
			foreach($users as $u) if($u === "*") return(true);
			else $opers[] = $u;
		}
	}
}

/* These are all layer 2 functions. */

/* This function fetches all objects of the selected type
 * It can optionally take an array of files, and will then only retrieve
 * objects that goes into those files. This argument must not be an empty
 * array.
 */
function get_objects($obj_type, $for_files=null) {
	global $DEBUG;

	$table = $obj_type;
	$order = " ORDER BY register";
	$where = false;
	# do some extra fiddling to get sorting right
	switch ($obj_type) {
	 case 'host_template':
		$table = 'host';
		$where = " register = 0 AND host_name IS NULL";
		# fallthrough
	 case 'host':
		$order .= ', name, host_name';
		break;

	 case 'service_template':
		$table = 'service';
		$where = " register = 0 AND host_name IS NULL AND service_description IS NULL";
		# fallthrough
	 case 'service':
		$order .= ', name, host_name, service_description';
		break;

	 case 'hostdependency':
		$order .= ', dependent_host_name, host_name';
		break;

	 case 'servicedependency':
		$order .= ', dependent_service, service';
		break;

	 case 'hostescalation':
		$order .= ', host_name';
		break;

	 case 'serviceescalation':
		$order .= ', service';
		break;

	 case 'command':
		$order = ' ORDER BY command_name';
		break;

	 case 'contact_template':
		$table = 'contact';
		$order .= ', name';
		$where = ' contact_name = 0';
		break;

	 case 'timeperiod': case 'hostgroup': case 'servicegroup':
	 case 'contact': case 'contactgroup':
		$order .= ', ' . $obj_type . '_name';
		break;

	 case 'FILE':
		$table = 'file_tbl';
		$order = '';
		break;

	 # listed for clarity
	 case 'hostextinfo': case 'serviceextinfo':
	 default:
		$order = "";
		break;
	}

	$query = "SELECT $table.* FROM $table ";
	if ($for_files) {
		$in_str = '(';
		foreach ($for_files as $file) {
			$in_str .= pdo_quote_string($file).',';
		}
		$in_str[strlen($in_str)-1] = ')';
		$query .= " INNER JOIN file_tbl f ON $table.file_id = f.id " .
		          " AND f.file_name IN $in_str ";
	}

	if ($where)
		$query .= " WHERE " . $where;
	$query .= $order;

	$result = pdo_exec_query($query);
	$res = pdo_fetch_result($result);
	unset($result);
	if (!empty($res)) {
		$p = get_object_properties($obj_type);
		foreach ($p['special'] as $key => $type) {
			$obj_var = $table;
			if ($p['var_index'][$key] !== 'mselect')
				continue;
			$jc = get_junction_table_name($obj_var, $key);
			if($obj_type === 'host' && $key === 'children') {
				$obj_var = 'parents';
			} else if (($obj_type === 'host' && $key === 'parents') || ($obj_type === 'timeperiod' && $key === 'exclude')) {
				$type = $key;
			} else if (substr($key, -13) === 'group_members') {
				$type = 'members';
			}

			$query = "SELECT $obj_var AS id, $type FROM $jc";
			$result = pdo_exec_query($query);
			while ($rel = $result->fetch(PDO::FETCH_ASSOC)) {
				if (!isset($res[$rel['id']]))
					continue;
				if (!isset($res[$rel['id']][$key]))
					$res[$rel['id']][$key] = array($rel[$type]);
				else
					$res[$rel['id']][$key][] = $rel[$type];
			}
			unset($result);
		}
	}
	return($res);
}

/**
 * Fetches custom vars for the $type object with id $id
 */
function get_custom_vars($obj_type, $obj_id)
{
	$custom_table = 'custom_vars';
	if ($obj_type === 'management_pack')
		$custom_table = 'persistent_custom_vars';
	$query = "SELECT * FROM $custom_table " .
		'WHERE obj_type = '.pdo_quote_string($obj_type).' AND obj_id = '.pdo_quote_string($obj_id);
	$result = pdo_exec_query($query);
	$ret = array();
	while (($ary = $result->fetch(PDO::FETCH_ASSOC)))
		$ret[$ary['variable']] = $ary['value'];
	unset($result);
	return $ret;
}

/* This baby handles populating the required index-tables and
 * multi-referencing variables (reading junction data).
 * Since index tables are cached internally, we can happily
 * ignore the fact that we might be fetching the same index multiple
 * times (although a list could be created to save us some function calls) */
function get_jnc_object_data($obj_type, $obj_list) {
	global $DEBUG;

	// if there are no objects, return immediately
	if(empty($obj_list)) return($obj_list);

	$props = get_object_properties($obj_type);
	$v_index = $props['var_index'];
	$spec = $props['special'];
	$junction_list = get_junction_list($obj_type);

	// not all objects refer to junctions
	if(empty($junction_list)) return($obj_list);

	foreach($obj_list as $obj_key => $obj) {
		// 'new' objects can't have junction data yet, so skip
		if($obj_key === 'new') continue;
		foreach($junction_list as $obj_var => $ref_obj_type) {
			// don't overwrite junctions already set (from Apply)
			if(!empty($obj[$obj_var])) continue;

			// fetch the data
			get_object_index_table($ref_obj_type);
			$jnc_data = get_junction_data($obj_type, $obj_key, $obj_var, $ref_obj_type);
			if(!empty($jnc_data)) {
				$obj[$obj_var] = $jnc_data;
			}
		}

		$obj_list[$obj_key] = $obj;
	}

	return($obj_list);
}

/* returns the proper data-structure for the object type in question
 * warning: do not under any condition use for any purpose ever - ask
 * the object class.
 */
function get_object_properties($obj_type) {
	global $Object_Properties;

	if(isset($Object_Properties[$obj_type])) {
		return($Object_Properties[$obj_type]);
	}
	else if (strpos($obj_type, '_template') !== false && isset($Object_Properties[substr($obj_type, 0, -strlen('_template'))])) {
		return $Object_Properties[substr($obj_type, 0, -strlen('_template'))];
	}
	else {
		return(false);
	}
}

/**
 * @name	glue_custom_vars
 * @param	object type (string)
 * @param	object id (integer)
 * @param	array (custom variables, k => v style)
 */
function glue_custom_vars($obj_type, $obj_id, $custom)
{
	global $IMPORTING;

	$ret = true;

	# optimize for when we're importing, as we can safely
	# avoid running tens of thousands of queries that way
	if ($IMPORTING && empty($custom))
		return true;

	$esc_obj_type = pdo_quote_string($obj_type);
	$esc_obj_id = pdo_quote_string($obj_id);
	$custom_table = 'custom_vars';
	if ($obj_type === 'management_pack')
		$custom_table = 'persistent_custom_vars';
	$purge_query = 'DELETE FROM '.$custom_table.' WHERE ' .
		"obj_type = $esc_obj_type AND obj_id = $esc_obj_id";

	$result = pdo_exec_query($purge_query);
	if (!$custom)
		return $result;

	$base_query = "INSERT INTO $custom_table VALUES(" .
		$esc_obj_type . "," . $esc_obj_id . ", ";

	foreach ($custom as $k => $v) {
		$query = $base_query . pdo_quote_string($k) . "," .
			pdo_quote_string($v) . ")";
		$result = pdo_exec_query($query);
		if (!$result)
			$ret = false;
		unset($result);
	}

	return $ret;
}

/* takes an indexed array of associative arrays as input and writes them
 * to database. This is the function every script that modifies objects
 * MUST use to glue their objects back together, once they have been
 * normalised. It also handles the junction tables and ensures the
 * database integrity.
 */
function glue_objects($obj_list, $table_name) {
	global $DEBUG;
	global $SUPPORT;
	global $IMPORTING;

	# no objects, so we're done glueing
	if(empty($obj_list)) return true;

	$db_obj_insert = false;
	$Junctions = false;
	$delayed_inserts = false;
	$p = get_object_properties($table_name);
	$v_index = $p['var_index']; // let's recognise checkbox vars properly
	$spec = $p['special'];

	# build the list of junction tables we need to delete references from
	$jnc_list = array();
	if(!$IMPORTING) {
		$jnc_list = get_junction_list($table_name);
		if($table_name === 'host') {
			unset($jnc_list['parents']);
			unset($jnc_list['children']);
		}
	}

	// for debugging purposes
	$db_obj_insert[$table_name] = 0;
	$db_obj_insert['real'][$table_name] = 0;

	# hoist this out of the loop. one hash-lookup is much faster than
	# several if() branches when there are many objects (when there are few
	# it doesn't matter anyway)
	if(isset($obj_list['new'])) {
		$nid = get_next_object_id($table_name);
		$obj_list[$nid] = $obj_list['new'];
		unset($obj_list['new']);
	}

	foreach(array_keys($obj_list) as $obj_key) {
		if ($table_name == 'FILE') {
			pdo_exec_query("DELETE FROM file_tbl WHERE id=$obj_key");
		}
		else {
			pdo_exec_query("DELETE FROM $table_name WHERE id=$obj_key");
		}
	}

	# commands are inherently simple (no out-references, no junctions, no
	# checkboxes), so we handle those separately for speed. They are also
	# always committed in bulk (up to 100 at a time), so it's a reasonable
	# simplicity / performance trade-off
	if($table_name === 'command') {
		$stmt = pdo_prepare_statement("INSERT INTO command VALUES(:id, :name, :line, 1, :fileid)");
		foreach($obj_list as $obj_key => $obj) {
			$stmt->execute(array('id' => $obj_key, 'name' => $obj['command_name'], 'line' => $obj['command_line'], 'fileid' => $obj['file_id']));
		}
		return true;
	}

	# FILE objects are even simpler, so we handle those separately
	# as well. This also avoids the whole issue of FILE being a
	# reserved word in Oracle.
	if ($table_name === 'FILE') {
		foreach ($obj_list as $obj_key => $obj) {
			pdo_exec_query("INSERT INTO file_tbl VALUES('$obj_key', " .
						   pdo_quote_string($obj['file_name']) . ")");
		}
		return true;
	}

	# not a command, so do the complex loop
	foreach($obj_list as $obj_key => $obj) {
		# only handle numerical object_keys. We really should print some sort
		# of 'hack-atack' statement here, but the fact that it just doesn't
		# work might be a deterrant as well.
		if(!is_numeric($obj_key)) continue;
		$obj['id'] = $obj_key;

		# FILE is a reserved word in at least oracle, so we must
		# rename it to avoid running into sql errors
		if (isset($obj['FILE'])) {
			$obj['file_id'] = $obj['FILE'];
			unset($obj['FILE']);
		}

		# purge all junction references indiscriminately.
		# They will be re-added later on if they're still there, and this
		# way we save ourselves a couple of cmp/branch
		if($table_name === 'host' && !$IMPORTING) {
			# Note that we can't delete 'the other end' for parents and
			# such, or we would force users to have their hosts configured
			# in reverse order of their network topology (bug #181).
			# This doesn't present a problem so long as we remember
			# to purge it when *deleting* a host
			pdo_exec_query("DELETE FROM host_parents " .
						   "WHERE host = $obj_key");
			# we can if it's either not empty or not set though though,
			# since it will be auto-updated if it is, and only new children
			# can be added (none removed) unless we do.
			if (!empty($obj['children']) || !isset($obj['children']))
			  pdo_exec_query("DELETE FROM host_parents " .
							 "WHERE parents = $obj_key");
		}
		if (!$IMPORTING && !empty($jnc_list) && is_array($jnc_list)) {
			foreach($jnc_list as $k => $v) {
				$jnc = get_junction_table_name($table_name, $k);
				pdo_exec_query("DELETE FROM $jnc WHERE $table_name = $obj_key");
			}
		}

		// loop every variable in the object
		foreach($obj as $k => $v) {
			if(empty($v) && $v !== '0' && $v !== 0) {
				unset($obj[$k]);
			}
			// all arrays that are still around require junctions
			// we know this since we've uniqueified every other object
			elseif (is_array($v)) {
				if($v_index[$k] === 'checkbox') {
					$obj[$k] = stringify_checkbox($v);
					continue;
				}

				// not a checkbox var, so it must be a junction.
				unset($obj[$k]);
				$query = false;
				$junction = get_junction_table_name($table_name, $k);

				# set up the values and insert them
				# All junctions are indiscriminately purged above, so
				# it should be safe to do this now.
				foreach($v as $junc_part) {
					if(empty($junc_part) || $junc_part === '0') {
						echo "SQL WARNING!! Inserting empty values in $junction. type: $table_name, var: $k, key: $obj_key\n";
						echo "This is an index normalisation bug. Please contact $SUPPORT so that this issue may be resolved.<br />\n";
					}
					$values = pdo_quote_string($obj_key) . ', ' . pdo_quote_string($junc_part);
					$other_obj_type = $spec[$k];
					// hosts require special handling
					$query = false;
					if ($k === 'children') {
						$query = "INSERT INTO host_parents " .
						  "(parents, host) VALUES($values)";
					}
					elseif (substr($junction, -13) === 'group_members') {
						$query = "INSERT INTO $junction " .
							"($table_name, members) VALUES($values)";
					}
					elseif ($other_obj_type === $table_name) {
						# self-multi-referencing object.
						# 'timeperiod_exclude' or 'host_parents'
						$other_obj_type = $k;
					}

					if($query === false) {
						if (empty($obj_key)) {
							$delayed_inserts[] = "INSERT INTO $junction " .
							  "(" . $table_name . ',' . $other_obj_type . ') ' .
						   "VALUES(%s, ".pdo_quote_string($junc_part).")";
						} else {
							$query = "INSERT INTO $junction " .
							  "(" . $table_name . ',' . $other_obj_type . ') ' .
						   "VALUES($values)";
						}
					}
					if (!empty($query))
						pdo_exec_query($query);
				}
				continue;
			}
		}

		// ok... obj has been purged of all evil arrays, and strings are
		// properly escaped. It *should* be safe to insert it into the db now.
		$db_obj_insert[$table_name]++;
		$target_vars = implode(',', array_keys($obj));
		$target_values = ':'.implode(', :', array_keys($obj));
		$query = "INSERT INTO $table_name($target_vars) " .
		  "VALUES($target_values)";
		$stmt = pdo_prepare_statement($query);
		try {
			$result = $stmt->execute($obj);
		} catch (PDOException $ex) {
			pdo_print_error($ex);
		}


		# if we are inserting a new object we don't have the id to use
		# for junction data yet. The queries that should be committed are then
		# stored in $delayed_inserts and can be built with a simple sprintf()
		if (!empty($delayed_inserts)) {
			$new_id = pdo_last_insert_id($table_name);
			if (!empty($new_id)) {
				foreach ($delayed_inserts as $sql) {
					pdo_exec_query(sprintf($sql, $new_id));
				}
			}
		}
		$db_obj_insert['real'][$table_name]++;
		$obj_array[$table_name][$obj_key] = $obj;
	}
	return true;
}

function get_junction_table_name($obj_type, $v_name) {
	if($obj_type . 'groups' === $v_name) {
		return($obj_type . '_' . $obj_type . 'group');
	}
	if($v_name . 'group' === $obj_type) {
		return($v_name . '_' . $v_name . 'group');
	}
	if($v_name === 'members') {
		return(substr($obj_type, 0, -5) . '_' . $obj_type);
	}
	if($v_name === 'contact_groups') {
		return($obj_type . '_contactgroup');
	}
	if ($v_name === 'contacts')
		return $obj_type . '_contact';

	if($v_name === 'children') {
		return('host_parents');
	}
	if(substr($v_name, -13) === 'group_members')
		return $v_name;

	return($obj_type . '_' . $v_name);
}

function get_object_name_by_array($obj_type, $obj) {
	global $Object_Index_Table;
	if (!isset($obj['id']) || !$obj['id'])
		$obj['id'] = 'new';

	$friendly_type = str_replace('_t', ' T', ucfirst($obj_type));

	if ($obj['id'] === 'new')
		$friendly_type = 'New ' . $friendly_type;

	if (count($obj) === 1)
		return $friendly_type;

	if (isset($obj['name']) && $obj['name']) {
		return $obj['name'];
	}

	switch ($obj_type) {
	 case 'FILE':
	 case 'service':
	 case 'service_template':
	 case 'servicegroup':
	 case 'host':
	 case 'host_template':
	 case 'hostgroup':
	 case 'contact':
	 case 'contact_template':
	 case 'contactgroup':
	 case 'timeperiod':
	 case 'command':
	 case 'management_pack':
		if ($obj_type == 'service' && $obj['service_description']) {
			if (isset($obj['host_name']) && $obj['host_name']) {
				$host_name = get_object_name_by_id('host', $obj['host_name']);
				return $host_name.';'.$obj['service_description'];
			}
			else if (isset($obj['hostgroup_name']) && $obj['hostgroup_name']) {
				$hostgroup_name = get_object_name_by_id('hostgroup', $obj['hostgroup_name']);
				return $hostgroup_name.';'.$obj['service_description'];
			}
		}
		else if (isset($obj[$obj_type.'_name']) && $obj[$obj_type.'_name']) {
			return $obj[$obj_type.'_name'];
		}
		else if (isset($obj['name']) && $obj['name']) {
			return $obj['name'];
		}

		get_object_index_table($obj_type);
		if (isset($Object_Index_Table[$obj_type][$obj['id']])) {
			return $Object_Index_Table[$obj_type][$obj['id']];
		}
		return $friendly_type;
		break;
	 case 'servicedependency':
		if (empty($obj['service']) || empty($obj['dependent_service']))
			$name = $friendly_type;
		else
			$name = '"'.get_object_name_by_id('service', $obj['dependent_service']).'" depends on "'.get_object_name_by_id('service', $obj['service']).'"';
		break;
	 case 'hostdependency':
		get_object_index_table('host_template');
		$master_type = !empty($obj['dependent_hostgroup_name']) ? 'hostgroup' : 'host';
		$name = '"'.get_object_name_by_id($master_type, $obj['dependent_'.$master_type.'_name']).'"';
		$deps = array();
		if (isset($obj['host_name']) && $obj['host_name'] && isset($Object_Index_Table['host_template'][$obj['host_name']]))
			$deps[] = 'host template "'.get_object_name_by_id('host', $obj['host_name']).'"';
		else if (isset($obj['host_name']) && $obj['host_name'])
			$deps[] = 'host "'.get_object_name_by_id('host', $obj['host_name']).'"';
		if (isset($obj['hostgroup_name']) && $obj['hostgroup_name'])
			$deps[] = 'hostgroup "'.get_object_name_by_id('hostgroup', $obj['hostgroup_name']).'"';
		if (!empty($deps))
			$name .= ' depends on '.implode(' and ', $deps);
		else
			$name = $friendly_type . ' for ' . $name;
		break;
	 case 'serviceescalation':
		$name = 'Service Escalation';
		if (isset($obj['service']) && $obj['service'])
			$name .= ' for '.get_object_name_by_id('service', $obj['service']);
		else
			$name = 'New ' . $name;
		break;
	 case 'hostescalation':
		$name = 'Host Escalation';
		if (isset($obj['host_name']) && $obj['host_name'])
			$name .= ' for host ' . get_object_name_by_id('host', $obj['host_name']);
		else if (isset($obj['hostgroup_name']) && $obj['hostgroup_name'])
			$name .= ' for hostgroup ' . get_object_name_by_id('hostgroup', $obj['hostgroup_name']);
		else
			$name = 'New ' . $name;
		break;
	 default:
		if (isset($obj[$obj_type.'_name']) && !empty($obj[$obj_type.'_name']))
			$name = $obj[$obj_type.'_name'];
		else if ($obj['id'] !== 'new')
			$name = $friendly_type.' #'.$obj['id'];
		else
			$name = $friendly_type;
		break;
	}
	return $name;
}

function get_object_name_by_id($obj_type, $obj_id) {
	try {
		$fields = array('id');
		switch ($obj_type) {
		 case 'FILE':
		 case 'service':
		 case 'service_template':
		 case 'servicegroup':
		 case 'host':
		 case 'host_template':
		 case 'hostgroup':
		 case 'contact':
		 case 'contact_template':
		 case 'contactgroup':
		 case 'timeperiod':
		 case 'command':
		 case 'management_pack':
			global $Object_Index_Table;
			get_object_index_table($obj_type);
			if (isset($Object_Index_Table[$obj_type][$obj_id]))
				return $Object_Index_Table[$obj_type][$obj_id];
			else
				return "New ".str_replace('_t', ' T', ucfirst($obj_type));
			break;
		 case 'servicedependency':
			$fields[] = 'dependent_service';
			/* fallthrough */
		 case 'serviceescalation':
			$fields[] = 'service';
			break;
		 case 'hostdependency':
			$fields[] = 'dependent_host_name';
			$fields[] = 'dependent_hostgroup_name';
			/* fallthrough */
		 case 'hostescalation':
			$fields[] = 'host_name';
			$fields[] = 'hostgroup_name';
			break;
		 default:
			if (!is_numeric($obj_id))
				return $obj_id; # Looks presentable enough, donnit?
			$fields[] = $obj_type . '_name';
			break;
		}
		$res = pdo_exec_query("select ".implode(',', $fields)." from $obj_type where id = $obj_id", false);
		$attribs = $res->fetchAll(PDO::FETCH_ASSOC);
		if (!empty($attribs))
			$attribs = $attribs[0];
		else
			$attribs = array();
		unset($res);
	} catch (PDOException $ex) {
		return false;
	}

	return get_object_name_by_array($obj_type, $attribs);
}

$Objects_For_User = array();
/**
 * Generate an index of the hosts/services a user is allowed to see
 *
 * This is equivalent to getting all objects for an obj_type, and filtering by
 * $obj->can_user_read()
 *
 * @returns array [id => object_name], or false
 */
function get_objects_for_user($obj_type, $write = false) {
	$log = Op5Log::instance('auth');
	global $Objects_For_User, $DEBUG;
	if (is_array($obj_type)) {
		return false;
	}

	if (isset($Objects_For_User[$obj_type])) {
		return $Objects_For_User[$obj_type];
	}

	$rules = get_nacoma_access($obj_type);

	// Please stop complaining PHP!
	if (!is_array($rules)) {
		$rules = array();
	}

	// Look for recognized rules, from highest-access to lowest
	if (in_array(NACOMA_ACCESS_FULL_ACCESS, $rules) ||
	         (!$write && in_array(NACOMA_ACCESS_VIEW_ALL, $rules)) ||
	         ($write && in_array(NACOMA_ACCESS_EDIT_ALL, $rules))) {
		return get_user_object_index_table($obj_type);
	}
	else if ((!$write && in_array(NACOMA_ACCESS_VIEW_CONTACT, $rules) && !in_array(NACOMA_ACCESS_VIEW_ALL, $rules)) || ($write && in_array(NACOMA_ACCESS_EDIT_CONTACT, $rules) && !in_array(NACOMA_ACCESS_EDIT_ALL, $rules))) {
		// Get a list of the objects a user can access
		$Objects_For_User[$obj_type] = get_auth_objects_contact_access($obj_type);
		return $Objects_For_User[$obj_type];
	}
	else {
		// Default action: deny
		if ($DEBUG && !empty($rules))
			print "<p>The rules ".implode(',', $rules)." supposedly apply to ".gui_class::escape_html(get_user())." for objects of type $obj_type, which I don't understand</p>";
		return array();
	}
}

/**
 * Recursively resolve group memberships of contact (= user)
 *
 * @param $user string
 * @return array of contactgroup IDs
 */
function get_auth_objects_contact_access($obj_type) {
	$user = pdo_quote_string(get_user());
	$get_template_tree = function($objname, $type) {
		$objects = array();
		$new_objects = pdo_exec_query("
			SELECT
				id, {$type}_name
			FROM
				{$type}
			WHERE
				{$type}_name = ".$objname." AND {$type}.id ".get_exclude_pending_string($type))
			->fetchAll(PDO::FETCH_KEY_PAIR);
		$safety_limit_depth = 30; // because while-true looks so guilty
		while($new_objects && --$safety_limit_depth) {
			foreach($new_objects as $key => $val)
				$objects[$key] = $val;
			$new_objects = pdo_exec_query("
				SELECT
					tmpl.id, tmpl.name
				FROM
					{$type} AS obj
				INNER JOIN
					{$type} AS tmpl
					 ON obj.template = tmpl.id
				WHERE
					obj.id IN (".implode(", ", array_keys($new_objects)).") AND tmpl.id ".get_exclude_pending_string($type))
				->fetchAll(PDO::FETCH_KEY_PAIR);
			$new_objects = array_diff($new_objects, $objects);
		}
		return $objects;
	};
	$get_all_objects = function($cgs, $cs, $type) {
		if ($type == 'service') {
			$name = 'concat(concat(ifnull(host.host_name, hostgroup.hostgroup_name), \';\'), obj.service_description)';
		} else {
			$name = 'obj.'.$type.'_name';
		}
		$query = "
			SELECT
				obj.id, $name AS {$type}_name, obj.name
			FROM
				{$type} as obj
			LEFT JOIN
				{$type}_contact
				ON {$type}_contact.$type = obj.id
			LEFT JOIN
				{$type}_contactgroup
				ON {$type}_contactgroup.$type = obj.id";
		if ($type == 'service')
			$query .= "
				LEFT JOIN host ON host.id = obj.host_name
				LEFT JOIN hostgroup ON hostgroup.id = obj.hostgroup_name
			";
		$query .= "
			WHERE
				".(!empty($cs)?"{$type}_contact.contact IN (".implode(', ', array_keys($cs)).")":"0=1")."
			OR
				".(!empty($cgs)?"{$type}_contactgroup.contactgroup IN (".implode(', ', array_keys($cgs)).")":"0=1");
		$new_objects = pdo_exec_query($query)->fetchAll(PDO::FETCH_NUM);
		$all_objects = array();
		$safety_limit_depth = 30; // because while-true looks so guilty
		while($new_objects && --$safety_limit_depth) {
			$templates = array();
			foreach ($new_objects as $ary) {
				if ($ary[2]) {
					$templates[] = $ary[0];
					$all_objects[$ary[0]] = $ary[2];
				}
				else {
					$all_objects[$ary[0]] = $ary[1];
				}
			}
			if (!$templates)
				break;
			$query = "
				SELECT
					obj.id, $name AS {$type}_name, obj.name
				FROM
					{$type} AS obj";
			if ($type == 'service')
				$query .= "
					LEFT JOIN host ON host.id = obj.host_name
					LEFT JOIN hostgroup ON hostgroup.id = obj.hostgroup_name
				";
			$query .= "
				WHERE
					obj.template IN (".implode(", ", $templates).")
					AND obj.id ".get_exclude_pending_string($type);
			$new_objects = pdo_exec_query($query)->fetchAll(PDO::FETCH_NUM);
		}
		return $all_objects;
	};
	$get_group_objects = function($type, $plains) {
		$objects = array();
		$new_objects = pdo_exec_query("
			SELECT
				${type}group.id, ${type}group.${type}group_name
			FROM
				${type}group
			INNER JOIN
				${type}_${type}group ON ${type}_${type}group.${type}group = ${type}group.id
			WHERE
				".(!empty($plains)?"${type}_${type}group.${type} IN (".implode(', ', array_keys($plains)).')':'0=1')."
				AND ${type}group.id ".get_exclude_pending_string("${type}group"))
			->fetchAll(PDO::FETCH_KEY_PAIR);
		$safety_limit_depth = 30; // because while-true looks so guilty
		while($new_objects && --$safety_limit_depth) {
			foreach($new_objects as $key => $val)
				$objects[$key] = $val;
			$new_objects = pdo_exec_query("
				SELECT
					${type}group.id, ${type}group.${type}group_name
				FROM
					${type}group
				INNER JOIN
					${type}group_members ON ${type}group_members.${type}group = ${type}group.id
				WHERE
					${type}group_members.members IN (".implode(', ', array_keys($new_objects)).")
					AND ${type}group.id ".get_exclude_pending_string("${type}group"))
				->fetchAll(PDO::FETCH_KEY_PAIR);
			$new_objects = array_diff($new_objects, $objects);
		}
		return $objects;
	};
	$contacts = $get_template_tree($user, 'contact');
	$cgs_for_user = $get_group_objects('contact', $contacts);

	switch ($obj_type) {
	 case 'host':
		$res = $get_all_objects($cgs_for_user, $contacts, 'host');
		return $res;
	 case 'service':
		$tmp = $get_all_objects($cgs_for_user, $contacts, 'service');
		$hosts = get_objects_for_user('host');
		$hostgroups = get_objects_for_user('hostgroup');
		$res = pdo_exec_query("
			SELECT
				s.id, concat(concat(ifnull(host.host_name, hostgroup.hostgroup_name), ';'), s.service_description)
			FROM
				service as s
			LEFT JOIN host ON host.id = s.host_name
			LEFT JOIN hostgroup ON hostgroup.id = s.hostgroup_name
			WHERE
				".(!empty($hosts)?"s.host_name IN (".implode(', ', array_keys($hosts)).")":"0=1")."
			OR
				".(!empty($hostgroups)?"s.hostgroup_name IN (".implode(', ', array_keys($hostgroups)).")":"0=1")."
			AND s.id ".get_exclude_pending_string('service'))
			->fetchAll(PDO::FETCH_KEY_PAIR);
		foreach ($tmp as $k => $v)
			$res[$k] = $v;
		return $res;
	 case 'contact':
		// The 5.4+ behaviour was to retrieve any contact on a host or service
		// I could access - keep doing that, but add anything I'm in.
		$services = get_objects_for_user('service');
		$hosts = get_objects_for_user('host');
		$res = pdo_exec_query("
			SELECT
				c.id, c.contact_name
			FROM
				contact c
			LEFT JOIN host_contact hc ON hc.contact = c.id LEFT JOIN host h ON h.id = hc.host
			LEFT JOIN service_contact sc ON sc.contact = c.id LEFT JOIN service s ON s.id = sc.service
			WHERE
				".(!empty($hosts)?"h.id IN (".implode(', ', array_keys($hosts)).")":"0=1")."
			OR
				".(!empty($services)?"s.id IN (".implode(', ', array_keys($services)).")":"0=1"))
			->fetchAll(PDO::FETCH_KEY_PAIR);
		foreach ($contacts as $k => $v)
			$res[$k] = $v;
		return $res;
	 case 'hostgroup':
		$hosts = get_objects_for_user('host');
		return $get_group_objects('host', $hosts);
	 case 'servicegroup':
		$services = get_objects_for_user('service');
		return $get_group_objects('service', $services);
	 case 'contactgroup':
		// The 5.4+ behaviour was to retrieve any contactgroup on a host or service
		// I could access - keep doing that, but add anything I'm in.
		$services = get_objects_for_user('service');
		$hosts = get_objects_for_user('host');
		$res = pdo_exec_query("
			SELECT
				c.id, c.contactgroup_name
			FROM
				contactgroup c
			LEFT JOIN host_contactgroup hc ON hc.contactgroup = c.id LEFT JOIN host h ON h.id = hc.host
			LEFT JOIN service_contactgroup sc ON sc.contactgroup = c.id LEFT JOIN service s ON s.id = sc.service
			WHERE
				".(!empty($hosts)?"h.id IN (".implode(', ', array_keys($hosts)).")":"0=1")."
			OR
				".(!empty($services)?"s.id IN (".implode(', ', array_keys($services)).")":"0=1"))
			->fetchAll(PDO::FETCH_KEY_PAIR);
		foreach ($cgs_for_user as $k => $v)
			$res[$k] = $v;
		return $res;
	 default:
		return array();
	}

	$result = pdo_exec_query($query);
	if (!$result)
		return array();

	// fetchAll() might return false..
	$result = $result->fetchAll(PDO::FETCH_KEY_PAIR);

	return $result ? $result : array();
}

function get_user_object_index_table($obj_type) {
	$exclude_pending_query = 'id '.get_exclude_pending_string($obj_type);

	$query = false;
	$db_table = $obj_type;
	switch ($obj_type) {
	 case 'FILE':
		$db_table = 'file_tbl';
		# fallthrough
	 case 'host': case 'command': case 'contact': case 'timeperiod':
	 case 'servicegroup': case 'hostgroup': case 'contactgroup':
		$query = "SELECT id, ${obj_type}_name FROM $db_table " .
			"WHERE ${obj_type}_name IS NOT NULL " .
			"AND $exclude_pending_query " .
			"ORDER BY ${obj_type}_name";
		break;
	 case 'management_pack':
		$query = "SELECT ${obj_type}.id, ${obj_type}_name FROM $db_table " .
			"INNER JOIN hostgroup ON hostgroup_name = hostgroup " .
			"WHERE ${obj_type}_name IS NOT NULL " .
			"AND ${obj_type}.$exclude_pending_query " .
			"ORDER BY ${obj_type}_name";
		break;
	 case 'host_template': case 'contact_template': case 'service_template':
		$query = "SELECT id, name " .
			"FROM " . substr($obj_type, 0, -strlen('_template')) . " " .
			"WHERE name IS NOT NULL " .
			"AND $exclude_pending_query " .
			"ORDER BY name";
		break;
	 case 'service':
		$query = "(SELECT service.id, " .
			"concat(concat(host.host_name, ';'), service.service_description) " .
			"AS service_name " .
			"FROM host, service " .
			"WHERE host.id = service.host_name " .
			"AND service.service_description IS NOT NULL " .
			"AND service.$exclude_pending_query " .
			") UNION ALL" .
			"(SELECT service.id, " .
			"concat(concat(hostgroup.hostgroup_name, ';'), service.service_description) " .
			"AS service_name " .
			"FROM hostgroup, service " .
			"WHERE hostgroup.id = service.hostgroup_name " .
			"AND service.service_description IS NOT NULL " .
			"AND service.$exclude_pending_query " .
			") ORDER BY service_name";
		break;
	 case 'hostdependency': case 'hostescalation':
	 case 'serviceescalation': case 'servicedependency':
		$query = "SELECT id, concat('".ucfirst($obj_type)." #', id) AS {$obj_type}_name " .
			"FROM $obj_type WHERE $exclude_pending_query ORDER BY {$obj_type}_name";
		break;
	 default:
		return false;
	}

	$result = pdo_exec_query($query);
	if (!$result)
		return(false);

	# return empty array if there were no objects to find
	$res = array();
	while($row = $result->fetch(PDO::FETCH_NUM)) {
		$res[$row[0]] = $row[1];
	}

	return $res;
}

// generate index of the required object type, and return it.
// we cache internally for obvious reasons.
$Object_Index_Table = false;
function get_object_index_table($obj_type) {
	global $Object_Index_Table;

	# if this index table is cached, we return immediately
	if (!is_array($obj_type) && isset($Object_Index_Table[$obj_type]))
		return($Object_Index_Table[$obj_type]);

	$query = false;
	$db_table = $obj_type;
	switch ($obj_type) {
	 case 'FILE':
		$db_table = 'file_tbl';
		# fallthrough
	 case 'host': case 'command': case 'contact': case 'timeperiod':
	 case 'servicegroup': case 'hostgroup': case 'contactgroup':
		$query = "SELECT id, ${obj_type}_name FROM $db_table " .
			"WHERE ${obj_type}_name IS NOT NULL " .
			"ORDER BY ${obj_type}_name";
		break;
	 case 'management_pack':
		$query = "SELECT ${obj_type}.id, ${obj_type}_name FROM $db_table " .
			"INNER JOIN hostgroup ON hostgroup_name = hostgroup " .
			"WHERE ${obj_type}_name IS NOT NULL " .
			"ORDER BY ${obj_type}_name";
		break;
	 case 'host_template': case 'contact_template': case 'service_template':
		$query = "SELECT id, name " .
			"FROM " . substr($obj_type, 0, -strlen('_template')) . " " .
			"WHERE name IS NOT NULL " .
			"ORDER BY name";
		break;
	 case 'service':
		$query = "(SELECT service.id, " .
			"concat(concat(host.host_name, ';'), service.service_description) " .
			"AS service_name " .
			"FROM host, service " .
			"WHERE host.id = service.host_name " .
			"AND service.service_description IS NOT NULL " .
			") UNION ALL" .
			"(SELECT service.id, " .
			"concat(concat(hostgroup.hostgroup_name, ';'), service.service_description) " .
			"AS service_name " .
			"FROM hostgroup, service " .
			"WHERE hostgroup.id = service.hostgroup_name " .
			"AND service.service_description IS NOT NULL " .
			") ORDER BY service_name";
		break;
	 case 'hostdependency': case 'hostescalation':
	 case 'serviceescalation': case 'servicedependency':
		$query = "SELECT id, concat('".ucfirst($obj_type)." #', id) AS {$obj_type}_name " .
			"FROM $obj_type ORDER BY {$obj_type}_name";
		break;
	 default:
		return false; # possibly print an error here
	}

	$result = pdo_exec_query($query);
	if (!$result)
		return(false);

	# return empty array if there were no objects to find
	$count = 0;
	while($row = $result->fetch(PDO::FETCH_NUM)) {
		$Object_Index_Table[$obj_type][$row[0]] = $row[1];
		++$count;
	}

	return $count ? $Object_Index_Table[$obj_type] : array();
}

/**
 * Given an object type and an object 'name', returns
 * the database id of that object.
 *
 * @return false if not found, a numerical id on success
 */
function get_object_index_id($obj_type, $name)
{
	$idx = get_object_index_table($obj_type);
	return array_search($name, $idx);
}

function get_junction_list($obj_type) {
	$jnc_ref = false;

	if(is_array($obj_type)) {
		bug("ERROR: get_junction_list(): obj_type is an array!", true);
	}
	$props = get_object_properties($obj_type);
	$spec = $props['special'];
	if(!$props) return(false);
	foreach($props['var_index'] as $k => $v) {
		if($v === 'mselect') $jnc_ref[$k] = $spec[$k];
	}

	return($jnc_ref);
}

function get_junction_data($obj_type, $obj_key, $obj_var, $jnc_var) {
	$jnc_data = false;

	$table = get_junction_table_name($obj_type, $obj_var);

	// deal with self-referencing objects (host_parents, *group_members for now)
	$query = false;
	if($obj_type === $jnc_var) $jnc_var = $obj_var;
	if($obj_type === 'host' && $obj_var === 'children') {
		$query = "SELECT host FROM host_parents WHERE parents=" . pdo_quote_string($obj_key);
	} else if (substr($obj_var, -13) === 'group_members') {
		$query = "SELECT members FROM $obj_var WHERE $obj_type=".pdo_quote_string($obj_key);
	}
	if($query === false) {
		$query = "SELECT $jnc_var FROM $table WHERE $obj_type =" . pdo_quote_string($obj_key);
	}
	$result = pdo_exec_query($query);
	while($row = $result->fetch(PDO::FETCH_NUM)) {
		$jnc_data[$row[0]] = $row[0];
	}
        unset($result);
	return($jnc_data?array_values($jnc_data):$jnc_data);
}


$Object_Reference_List = false;
function get_object_reference_list($obj_type) {
	global $Object_Reference_List;

	// use cache if available
	if(!empty($Object_Reference_List[$obj_type]))
	  return($Object_Reference_List[$obj_type]);

	$ret = false;
	if(is_array($obj_type)) {
		bug("ERROR: " . __FUNCTION__ . "(): obj_type is an array!", true);
	}
	$p = get_object_properties($obj_type);
	$spec = $p['special'];
	foreach($p['var_index'] as $k => $v) {
		if($v === 'select' || $v === 'mselect') {
			if(!array_key_exists($k, $spec)) continue;
			// extinfo objects have images in arrays
			if(is_array($spec[$k])) continue;
			$ret[$k] = $spec[$k];
			get_object_index_table($spec[$k]);
		}
	}

	// update cache
	$Object_Reference_List[$obj_type] = $ret;
	return($ret);
}

$Reverse_Object_Reference_Table = false;
function get_object_reverse_reference_list($master_obj_type) {
	global $Supported_Object_Types;
	global $Reverse_Object_Reference_Table;

	// use cache if available
	if(!empty($Reverse_Object_Reference_Table[$master_obj_type])) {
		return($Reverse_Object_Reference_Table[$master_obj_type]);
	}

	$ret = false;
	foreach($Supported_Object_Types as $obj_type) {
		$p = get_object_properties($obj_type);
		if(!isset($p['special'])) continue;
		$spec = $p['special'];
		foreach($spec as $k => $v) {
			if($v === $master_obj_type) $ret[$obj_type][] = $k;
		}
	}

	// update cache
	$Reverse_Object_Reference_Table[$master_obj_type] = $ret;
	return($ret);
}

// WARNING: you should never call this function unless you're just about to
// push stuff to the DB, as that will cause (more) collisions and crashes.
//
// this is a clumsy function to get highest possible id of an object.
// It should be left to the database instead, but then we'd get objects with
// ridiculously high ID's, which might cause DOS eventually
// OTOH, this is not concurrency-safe.
function get_next_object_id($obj_type) {
	$result = pdo_exec_query("SELECT id FROM $obj_type ORDER BY id DESC LIMIT 1");
	$row = $result->fetch(PDO::FETCH_NUM);
        unset($result);
	return $row ? ($row[0] + 1) : 1;
}

function get_num_hosts_configured() {
	$query = "SELECT count(host_name) AS num_hosts FROM host";
	$result = pdo_exec_query($query);
	$row = $result->fetch(PDO::FETCH_NUM);
	unset($result);
	if($row)
	  return $row[0];
	return 0;
}
