<?php
/*
 * Copyright(C) 2004, 2005, 2007, 2010 op5 AB
 * All rights reserved.
 *
 */

define("CUSTOM_DB_CONFIG", dirname(__FILE__)."/../custom_db_config.php");
include "oci8topdo.php";

/*
 * Database configuration
 */
$db_type = 'mysql'; // mysql or oracle
$db_host = 'localhost';
$db_user = 'nacoma';
$db_passwd = 'nacoma';
$db_database = 'nacoma';
$db_port = false;

// some database backends allow you to override the above with a custom
// connection string here:
$db_conn_str = false;

# variables settable from the CUSTOM_CONFIG files
if (is_file(CUSTOM_DB_CONFIG) && is_readable(CUSTOM_DB_CONFIG))
	require(CUSTOM_DB_CONFIG);

/**
  A singleton-like PDO database handle provider. It is intended to be used like this:

  $db = PDOProvider::db();

  That returns a shared PDO database object.

 */
class PDOProvider
{
	private static $dbs = array();
	private static $conf = array();
	/**
	  Returns a shared PDO database instance.  It establishes the
	  connection during the first call, and throws an error if
	  connection fails. On subsequent calls it will return the same
	  instance.

	  The $key parameter is used to figure out which configuration
	  information to use for the connection. The connection parameters
	  are taken from config($key), and that object must be populated by
	  the client before calling this.

	  To use a custom (non-default) configuration:

	  $c =& PDOFactory::config('someDbIdentifier');
	  $c['user'] = ...;
	  $c['passwd'] = ...;
	  $c['dsn'] = ...;
	  ...

	  Then:

	  $db = PDOFactory::db('someDbIdentifier');
	 */
	public static function db($key = 'default')
	{
		$db = @self::$dbs[$key];
		if( $db instanceof PDO ) return $db;
		$c =& self::config($key);
		if( !@is_array($c) )
		{
			throw new Exception("Database key [$key] has no configuration data. "
					."Use ".__CLASS__."::config(\"$key\") to set it up.");
		}
		if ($c['type'] == 'oracle' || $c['type'] == 'oci' || $c['type'] == 'ocilib') {
			$pdo = new Oci8ToPdo($c['host'], $c['database'], $c['user'], $c['passwd'], $c['port'], $c['conn_str']);
		} else {
			$attr = array( PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
				PDO::ATTR_AUTOCOMMIT => true,
				PDO::ATTR_PERSISTENT => false );
			$pdo = new PDO($c['dsn'], @$c['user'], @$c['passwd'], $attr);
			// mysql driver seems to ignore the attr argument above
			foreach ($attr as $k => $v) {
				$pdo->setAttribute($k, $v);
			}
		}
		return self::$dbs[$key] = $pdo;
	}


	/** Returns a reference to a configuration object (array) which
	  stores the db-related parameters for this class. $key is key
	  string suitable for passing to db(), and db() calls config($key)
	  to fetch its configuration data.

	  The returned reference is an array object, and the following array
	  keys are relevant for purposes of this class:

	  'type' = db type

	  'host', 'port' = the host name/port numbers for the db
	  server. The 'port' option is optional, and its default value
	  is backend-specific.

	  'user', 'passwd' = the db login credentials

	  'dbname' = the name of the db within the db server.


	  BE SURE, when modifying the returned object, that you catch it
	  by reference:

	  $foo =& PDOProvider::config();
	 */
	public static function & config($key='default')
	{
		$c =& self::$conf[$key];
		if( ! $c )
		{
			self::$conf[$key] = array();
			$c =& self::$conf[$key];
		}
		return $c;
	}
};

/* stuff the old-style globals into the PDOProvider's default configuration. */
$c =& PDOProvider::config();
$c['type'] = $db_type;
$c['host'] = $db_host;
$c['user'] = $db_user;
$c['passwd'] = $db_passwd;
$c['database'] = $db_database;
$c['port'] = $db_port;
$c['dsn'] = $db_conn_str;
if (!$c['dsn']) {
	$c['dsn'] = $db_type.':host='.$db_host.';dbname='.$db_database;
	if ($db_port)
		$c['dsn'] .= ';port='.$db_port;
}
$c['conn_str'] = $db_conn_str;
