<?php
require_once('config.inc.php');
require_once('changelog.inc.php');
require_once('autoscan.inc.php');
require_once(dirname(__FILE__).'/../classes/changelog_class.php');
require_once(dirname(__FILE__).'/../classes/object_class.php');
/**
 * Writes objects to configuration files in a format that
 * Nagios can read.
 * @param $obj_list Array with list of objects to write
 */
function glue_export($obj_list)
{
	$buf = '';

	$buf .= "###############################################################################\n";
	$buf .= "#  Generated by op5 Monitor webconfiguration exporter\n#\n";
	$buf .= "#  Exported " . date("Y-m-d H:i") . " by " .
		get_user() . "\n";
	$buf .= "#\n\n";
	$last_host = false;
	$last_hostgroup = false;
	foreach($obj_list as $obj_key => $obj) {
		$obj_type = $obj['obj_type'];

		# this shouldn't happen, but with clumsy callers
		# it might, so just skip those meta-objects
		if ($obj_type === 'FILE')
			continue;

		unset($obj['obj_type']);
		$p = get_object_properties($obj_type);

		$buf .= "\n";

		if($obj_type === 'service' &&
		   isset($obj['host_name']) &&
		   $last_host !== $obj['host_name']) {
			$last_host = $obj['host_name'];
			$last_hostgroup = false;
			$buf .= "####################################################\n" .
			  "#\n# Services for host $obj[host_name]\n#\n\n";
		}
		elseif($obj_type === 'service' &&
		       isset($obj['hostgroup_name']) &&
		       $last_hostgroup !== $obj['hostgroup_name']) {
			$last_hostgroup = $obj['hostgroup_name'];
			$last_host = false;
			$buf .= "####################################################\n" .
			  "#\n# Services for hostgroup $obj[hostgroup_name]\n#\n\n";
		}
		$buf .= "# $obj_type ";
		if(isset($obj['name'])) {
			$buf .= "template '$obj[name]'";
		}
		elseif(isset($obj['dependent_service_description']) && isset($obj['dependent_host_name']))  {
			$buf .= "for service '$obj[dependent_service_description]' on host '$obj[dependent_host_name]'";
		}
		elseif(isset($obj['dependent_service_description']) && isset($obj['dependent_hostgroup_name']))  {
			$buf .= "for service '$obj[dependent_service_description]' on hostgroup '$obj[dependent_hostgroup_name]'";
		}
		elseif(isset($obj['service_description']) && isset($obj['host_name']) && $obj_type !== 'service') {
			$buf .= "for service '$obj[service_description]' on host '$obj[host_name]'";
		}
		elseif(isset($obj['service_description']) && isset($obj['hostgroup_name']) && $obj_type !== 'service') {
			$buf .= "for service '$obj[service_description]' on hostgroup '$obj[hostgroup_name]'";
		}
		elseif(isset($obj['dependent_host_name'])) {
			$buf .= "for host '$obj[dependent_host_name]'";
		}
		elseif(isset($obj['dependent_hostgroup_name'])) {
			$buf .= "for hostgroup '$obj[dependent_hostgroup_name]'";
		}
		elseif($obj_type !== 'host' && isset($obj['host_name']) && $obj_type !== 'service') {
			$buf .= "for host '$obj[host_name]'";
		}
		elseif($obj_type !== 'hostgroup' && isset($obj['hostgroup_name']) && $obj_type !== 'service') {
			$buf .= "for hostgroup '$obj[hostgroup_name]'";
		}
		elseif(isset($p['topic']) && isset($obj[$p['topic']])) {
			$buf .= "'" . $obj[$p['topic']] . "'";
		}
		$buf .= "\ndefine $obj_type{\n";

		// fix service slave objects pretty printing, and make sure templates
		// always goes to the top.
		$v_index = array('use' => false);
		if(isset($p['var_index']['service'])) {
			$v_index['host_name'] = false;
			$v_index['hostgroup_name'] = false;
			$v_index['service_description'] = false;
		}
		if(isset($p['var_index']['dependent_service'])) {
			$v_index['dependent_host_name'] = false;
			$v_index['dependent_hostgroup_name'] = false;
			$v_index['dependent_service_description'] = false;
		}
		// these have different names in nagios compared to in nacoma.
		if($obj_type == 'contact') {
			$v_index['service_notification_commands'] = false;
			$v_index['host_notification_commands'] = false;
		}
		# done as two loops, as it results in custom variables being written
		# at the end, which looks better
		$v_index = array_merge(array_keys($v_index), array_keys($p['var_index']));
		foreach($v_index as $k) {
			if (!isset($obj[$k]) || $k === 'FILE')
				continue;
			$buf .= sprintf("    %-30s %s\n", $k, $obj[$k]);
			unset($obj[$k]);
		}
		foreach($obj as $k => $v) {
			if ($k === 'FILE')
				continue;
			$buf .= sprintf("    %-30s %s\n", $k, $v);
		}

		$buf .= "    }\n";
	}

	return $buf;
}

/**
 * Check if an array of files are writable or not
 * @param $fnames Array of filenames to check
 * @return 0 if all paths are writable, array of failed files otherwise.
 */
function unwritable_paths($fnames)
{
	global $HOME;
	$non_writable = array();
	foreach ($fnames as $f) {
		if (!posix_access($HOME.$f, POSIX_W_OK) && !posix_access($f))
			$non_writable[] = $f;
	}

	return $non_writable;
}

/**
 * Sorts $obj_array into a new array based on which file
 * each object should go in.
 * @return false if any object lacks a 'FILE' variable. Array otherwise.
 */
function obj_array_file_sort($obj_array, $changed_files)
{
	$f_array = array();
	// if the change renders a file empty, we must still set the array key
	foreach ($changed_files as $file)
		$f_array[$file] = array();

	foreach ($obj_array as $obj_type => $obj_list) {
		if (empty($obj_list)) {
			continue;
		}

		foreach($obj_list as $obj_key => $obj) {
			// no file to write object, critical fail
			if (!isset($obj['file_id']))
				return false;
			// file to write object isn't in changed files, critical fail
			if (!isset($f_array[$obj['file_id']]))
				return false;

			// temporarily set object type inside object
			$obj['obj_type'] = $obj_type;
			if (isset($obj['register']) && $obj['register'] === '1') {
				unset($obj['register']);
			}
			if (isset($obj['members']))
				unset($obj['members']);

			if (isset($obj['template'])) {
				$obj['use'] = $obj['template'];
				unset($obj['template']);
			}

			// we needed to rename these to fit them in the database.
			// rename them back again.
			if ($obj_type == 'contact') {
				if (isset($obj['service_notification_cmds'])) {
					$obj['service_notification_commands'] = $obj['service_notification_cmds'];
					unset($obj['service_notification_cmds']);
				}
				if (isset($obj['host_notification_cmds'])) {
					$obj['host_notification_commands'] = $obj['host_notification_cmds'];
					unset($obj['host_notification_cmds']);
				}
			}

			$f = $obj['file_id'];
			unset($obj['file_id']);

			$f_array[$f][] = $obj;
			unset($obj_array[$obj_type][$obj_key]);
		}
	}
	return $f_array;
}

/**
 * Fetches all objects from all tables in the database, complete
 * with junction data.
 * @returns An array of the fetched objects.
 */
function fetch_all_objects($for_files)
{
	global $Supported_Object_Types;

	foreach($Supported_Object_Types as $obj_type) {
		get_object_index_table($obj_type);
		if($obj_type === 'FILE') continue;
		$obj_list = get_objects($obj_type, $for_files);

		# unset 'children' variable. Nagios doesn't grok it
		if($obj_type === 'host' && is_array($obj_list)) {
			foreach($obj_list as $obj_key => $obj) {
				if(!isset($obj['children'])) continue;
				unset($obj['children']);
				$obj_list[$obj_key] = $obj;
			}
		}
		$obj_array[$obj_type] = $obj_list;
	}

	return $obj_array;
}

/**
 * Writes the by-file sorted object array to their respective
 * files. Files that were previously used and are now empty
 * are truncated to prevent stale objects with now-bogus
 * object-references from remaining.
 * @param $f_array by-file sorted object array
 */
function export_write_files($f_array)
{
	global $HOME;

	# write the stuff back to its files
	foreach($f_array as $fname => $obj_list) {
		if ($fname{0} !== '/')
			$fname = "$HOME/$fname";
		write_file($fname, glue_export($obj_list));
	}
}

/**
 * Update the changelog with time, date and name of export
 * @param $user Name of user doing the update
 */
function export_update_changelog($user = false)
{
	if (!$user)
		$user = get_user();

	pdo_exec_query("INSERT INTO action_log(username, action) VALUES(" .
                       pdo_quote_string($user) . ", 'export')");
}

/**
 * Helper for delete_config_backup
 * @param $path Path to delete recursively
 */
function recursive_delete($path, $keep_directory=false)
{
	$path = rtrim($path, '/');
	if (!file_exists($path))
		return false;

	if (is_file($path))
		return unlink($path);

	foreach (glob($path . '/*') as $subpath) {
		recursive_delete($subpath);
	}
	if (!$keep_directory)
		rmdir($path);
}

/**
 * Cleans up the config backup directory
 *
 * Called before each export, and after each successful export.
 */
function delete_config_backup()
{
	global $Backup_Config_Dir;
	# in case this is not set, we shouldn't delete other files.
	# that wouldn't be appreciated.
	if (!$Backup_Config_Dir)
		return;
	recursive_delete($Backup_Config_Dir, true);
}

/**
 * Backup config to a temporary directory during the export.
 */
function backup_config($files)
{
	delete_config_backup();
	global $Backup_Config_Dir, $HOME;
	foreach ($files as $file) {
		$backup_subdir = dirname($file);
		if (!@mkdir(rtrim($Backup_Config_Dir, '/') . '/' . $backup_subdir, 0777, true))
			continue;
		if ($file[0] != '/')
			$file = rtrim($HOME, '/').'/'.$file;
		if (file_exists($file))
			copy($file, rtrim($Backup_Config_Dir, '/') . '/' . $backup_subdir . '/' . basename($file));
	}
}

/**
 * Restore backed up config in case of export verification errors.
 */
function restore_backup($files)
{
	global $Backup_Config_Dir, $HOME;
	foreach ($files as $file) {
		// store a copy since the variable('s path) will be modified
		$submitted_filename = $file;
		$backup_subdir = dirname($file);
		if ($file[0] != '/') {
			$file = rtrim($HOME, '/').'/'.$file;
		}
		$directory_for_backups = rtrim($Backup_Config_Dir, '/').'/';
		if (file_exists($directory_for_backups. $submitted_filename)) {
			copy($directory_for_backups. $backup_subdir . '/' . basename($file), $file);
		}
	}
}

/**
 * Runs Nagios' configuration verification routines to ensure
 * that the new configuration is sane and can be loaded properly.
 * @return true on success, array on errors
 */
function exported_config_is_ok()
{
	global $Config;
	$cmd = '/usr/bin/asmonitor -q monitor -v '.$Config['main'];
	$verify = exec_cmd($cmd);
	return $verify['value'] === 0 ? true : $verify;
}

/**
 * Check to see if monitor/nagios is running.
 * @return true if yes, false if not
 */
function monitor_is_running()
{
	global $HOME;

	$lock_file = "$HOME/var/nagios.lock";
	if (!is_file($lock_file))
		return false;
	$buf = trim(file_get_contents($lock_file));
	if (empty($buf) || !is_numeric($buf))
		return false;

	$pid = intval($buf);
	if (!$pid)
		return false;

	exec("asmonitor kill -s 0 $pid 2>&1", $output, $ret);
	return $ret === 0;
}

/**
 * Try to reload monitor by sending a command to the command pipe
 * @return 0 on success. 1 if monitor isn't running. 2 if the command-pipe
 * is unwritable or doesn't exist.
 */
function export_reload_monitor()
{
	global $console_access, $HOME;
	# Try to reload monitor if it's running and its command-pipe is writable
	$cmd_pipe = $HOME.'/var/rw/nagios.cmd';
	$running = $fd = false;
	if (file_exists($cmd_pipe) === true && posix_access($cmd_pipe, POSIX_W_OK) === true) {
		$running = monitor_is_running();
		$fd = $running ? fopen($cmd_pipe, 'w') : false;
	}
	if($running === false)
		return 1;
	elseif($fd === false)
		return 2;

	else {
		$commit_time = time();
		$restart_time = $commit_time + 1;
		fwrite($fd, "[$commit_time] RESTART_PROGRAM;$restart_time\n");
		fclose($fd);
	}
	return 0;
}


/**
 * This is the main export function used by export.php
 *
 * @param $changes A changelog object to only export those changes, or TRUE to export everything
 *
 * @return TRUE if the config was successfully exported, and an array describing the error otherwise
 */
function do_export($changes)
{
	$l = op5Log::instance('nacoma');
	$l->log('debug', 'Starting a save');
	global $Supported_Object_Types;
	global $Object_Index_Table;
	global $SUPPORT, $DEBUG;
	if (!extend_lock('all', 0, 5*60)) {
		$l->log('warning', 'Trying to save, while save is already in progress');
		return array('type' => 'save in progress');
	}
	if ($changes !== true) {
		$overlay = array(); $changed_files = array();
		$changes->undo_others_changes($overlay, $changed_files);
		if (empty($changed_files)) {
			free_lock('all', 0);
			$l->log('debug', 'Trying to save, but nothing to save');
			return array('type' => 'nothing to do');
		}
	} else {
		$changed_files = array();
		$overlay = array();
		$res = pdo_exec_query('SELECT file_name FROM file_tbl');
		while ($row = $res->fetch(PDO::FETCH_ASSOC))
			$changed_files[] = $row['file_name'];
		unset($res);
	}
	$non_writable = unwritable_paths($changed_files);
	if ($non_writable) {
		free_lock('all', 0);
		$l->log('error', 'Trying to save, But files are unwritable: ' . implode(',', $non_writable));
		return array('type' => 'file write error', 'files' => $non_writable);
	}
	$obj_array = fetch_all_objects($changed_files);
	# merge the overlay with the fetched objects
	foreach ($overlay as $type => $objs) {
		if (substr($type, -strlen('_template')) == '_template')
			$type = substr($type, 0, -strlen('_template'));

		if (!isset($obj_array[$type])) {
			$obj_array[$type] = array();
		}
		foreach ($objs as $id => $obj) {
			if ($obj === false || $obj->obj === false) {
				unset($obj_array[$type][$id]);
				continue;
			}
			unset($obj->obj['id']);
			if($type === 'host' && isset($obj->obj['children']))
				unset($obj->obj['children']);
			$obj_array[$type][$id] = $obj->obj;
		}
	}

	get_object_index_table('host_template');
	get_object_index_table('service_template');
	get_object_index_table('contact_template');

	foreach($Supported_Object_Types as $obj_type) {
		if($obj_type === 'FILE') continue;

		// only treat objects that exists
		if(empty($obj_array[$obj_type])) {
			continue;
		}

		$props = get_object_properties($obj_type);
		$spec = $props['special'];
		$v_index = $props['var_index'];

		$obj_list = $obj_array[$obj_type];
		foreach($obj_list as $obj_key => $obj) {
			foreach($v_index as $k => $v) {
				# the 'members' directive is removed later, so
				# ignore it here or hostgroups/servicegroups
				# used with templates will complain about not
				# being able to locate some of their member objects
				if ($k === 'members')
					continue;

				// VALIDATE OBJECTS BEFORE WE GET TO THIS!!!
				if(!isset($obj[$k]))
					continue;

				// fix single references that aren't image refs.
				if(($v === 'select' && !is_array($spec[$k])) ||
				   ($v === 'hidden' && $k !== 'register')) {
					if(!isset($Object_Index_Table[$spec[$k]][$obj[$k]])) {
						unset($obj[$k]);
						continue;
					}
					else if (($obj_type === 'serviceextinfo' || $obj_type === 'servicedependency' || $obj_type === 'serviceescalation') && ($k === 'service' || $k === 'dependent_service')) {
						$prefix = (strpos($k, 'dependent') === 0) ? 'dependent_' : '';
						$str = explode(';', $Object_Index_Table[$spec[$k]][$obj[$k]], 2);
						$parent_type = service_class::get_parent_type($obj[$k]);
						$obj[$prefix.$parent_type.'_name'] = $str[0];
						$obj[$prefix.'service_description'] = $str[1];
						unset($obj[$k]);
					}
					else {
						$obj[$k] = $Object_Index_Table[$spec[$k]][$obj[$k]];
					}
					if($spec[$k] === 'command' && isset($obj[$k . '_args'])) {
						$obj[$k] .= '!' . $obj[$k . '_args'];
						unset($obj[$k . '_args']);
					}
				}
				elseif($v === 'mselect') {
					$tmp = false;
					if (is_array($obj[$k]))
						foreach($obj[$k] as $v_var) {
							if(isset($Object_Index_Table[$spec[$k]][$v_var]))
								$tmp[] = $Object_Index_Table[$spec[$k]][$v_var];
							else if ($DEBUG)
								echo "Object_Index_Table[ $spec[$k] ][ $v_var ] isn't set (obj_type is $obj_type)!<br />\n";
						}
					if(!empty($tmp))
						$obj[$k] = implode(',', $tmp);
					else
						unset($obj[$k]);
				}
				elseif($v === 'string') {
					# never export "_args" variables without
					# their corresponding command reference
					if (substr($k, -5) === '_args' && !isset($obj[substr($k, 0, -5)])) {
						unset($obj[$k]);
						continue;
					}
				}
			}
			$custom = get_custom_vars($obj_type, $obj_key);
			if (!empty($custom)) {
				$obj = array_merge($obj, $custom);
			}
			$obj_list[$obj_key] = $obj;
		}
		$obj_array[$obj_type] = $obj_list;
	}

	$f_array = obj_array_file_sort($obj_array, $changed_files);
	if ($f_array === false) {
		free_lock('all', 0);
		$l->log('error', 'Trying to save, but got ye olde evile FILE error');
		return array('type' => 'FILE');
	}
	backup_config($changed_files);
	export_write_files($f_array);
	export_update_changelog(get_user());

	// FIXME: This shortcut is incorrect - we need to recalculate this completely post-export
	// Validate autoscan results
	if ($changes !== true) {
		$hosts = $changes->get_host_list();
		foreach ($hosts as $host) {
			autoscan_result_visibility_change($host['address'], ($host['change'] == 'Add' ? 0 : 1));
		}
	}

	$verify = exported_config_is_ok();
	if($verify !== true) {
		restore_backup($changed_files);
		free_lock('all', 0);
		$l->log('warning', 'Trying to save, but preflight check failed');
		return array('type' => 'preflight check failed', 'output' => $verify['buf']);
	}

	delete_config_backup();

	# everything worked - let's run hooks
	require_once('hooks.inc.php');
	run_hooks('save');
	run_hooks('save_denormalized');

	if ($changes !== true) {
		$changes->delete_changes();
	} else {
		$c = new Changelog(null, null, true);
		$c->delete_changes();
	}

	free_lock('all', 0);
	$l->log('debug', 'Save completed');
	return true;
}
