<?php

require_once('changelog.inc.php');

function concat_shell_args($args)
{
	$ret = '';
	foreach ($args as $a) {
		if ($ret)
			$ret .= ' ';
		$ret .= escapeshellarg($a);
	}

	return $ret;
}

function glob_execs($path)
{
	if (!is_dir($path))
		return array($path);

	$ents = glob("$path/*");
	foreach ($ents as $k => $ent) {
		if (!is_file($ent) || !is_executable($ent) || 'README' == basename($ent))
			unset($ents[$k]);
	}

	return $ents;
}

function hookify_object($obj)
{
	$str = '';
	if (empty($obj))
		return $str;

	foreach ($obj as $k => $v) {
		if (is_array($v))
			$v = join(',', $v);
		$str .= "$k=$v;";
	}

	return $str;
}

function get_full_contact_name($user)
{
	return "unknown";
}

function run_hooks($type)
{
	global $HOOKDIR;
	$target = "$HOOKDIR/$type";

	if (!is_executable($target))
		return;

	$user = get_user();

	$args = false;
	$nargs = 1;
	switch ($type) {
	 case 'save':
	 case 'save_denormalized':
		$args[$nargs++] = $user;
		$args[$nargs++] = get_full_contact_name($user);
		break;
	 case 'test':
		$args[$nargs++] = 'test hook';
		break;
	}

	$args = concat_shell_args($args);
	$hooks = glob_execs($target);

	$query = 'SELECT * FROM changelog';
	$data = '';
	foreach ($hooks as $hook) {
		$ph = false;
		$result = pdo_exec_query($query);
		$pipes = false;
		while ($ary = $result->fetch(PDO::FETCH_ASSOC)) {
			if(false === $ph) {
				$ph = proc_open("$hook $args", array(array('pipe', 'r',), array('pipe', 'w'), array('pipe', 'w')), $pipes);
			}
			$ary_a = convert_changelog_object($ary['object_type'], $ary['oldstate']);
			$ary_b = convert_changelog_object($ary['object_type'], $ary['newstate']);
			if ($type == 'save_denormalized') {
				$class = $ary['object_type'] . '_class';
				$obj = new $class();
				$ary_a = $obj->denormalize($ary_a);
				$ary_b = $obj->denormalize($ary_b);
			}
			$pre_obj = hookify_object($ary_a);
			$post_obj = hookify_object($ary_b);
			unset($ary['oldstate']);
			unset($ary['newstate']);
			unset($ary['id']);
			$str = '';
			foreach ($ary as $k => $v)
				$str .= "$k='$v'\t";

			$str .= "pre='$pre_obj'\t";
			$str .= "post='$post_obj'";
			$str .= "\n";
			fwrite($pipes[0], $str);
		}

		if ($pipes !== false) {
			fclose($pipes[0]);
			// This assumes that as soon as STDIN is closed, the hook will go to work,
			// optionally write something to STDOUT, and then exit. A misbehaving hook
			// will lock up apache, so it would be sorta cool if you'd follow that.
			echo stream_get_contents($pipes[1]);
			echo stream_get_contents($pipes[2]);
		}
		unset($result);
		if(false !== $ph) {
			proc_close($ph);
		}
	}
}
