<?php
/*
 * Copyright(C) 2003-2005 op5 AB
 * All rights reserved.
 *
 */

/*
 * General Idea:
 * Each function using these data structures should call a pseudo-function
 * that simply returns the proper data.
 * In the database-supporting version, the pseudo-function should be replaced
 * by one fetching this precise information from the database instead.
 *
 * NOTE:
 *   'default' is used to auto-expand the objects that don't use
 *   templates, and also to add new objects when passed to the pretty_print
 *   function, in case $obj_list['new'] !== false.
 */

/* generally used arrays.
 * Fetched once here to reduce overhead */
$IMAGES = false;
$IMAGES[] = false;
$img_list = glob("/opt/monitor/op5/ninja/application/media/images/logos/*");
foreach($img_list as $img) $IMAGES[basename($img)] = basename($img);

/*****************************************************************************
 *                Data definitions for the 'host' object type                *
 *****************************************************************************/
$Object_Properties['host']['topic'] = 'host_name';
$Object_Properties['host']['var_index'] =
  array(
		'template' => 'select',
		'host_name' => 'string',
		'alias' => 'string',
		'display_name' => 'string',
		'address' => 'string',
		'hostgroups' => 'mselect',
		'parents' => 'mselect',
		'children' => 'mselect',
		'check_command' => 'select',
		'check_command_args' => 'string',
		'contacts' => 'mselect',
		'contact_groups' => 'mselect',
		'max_check_attempts' => 'int',
		'check_interval' => 'int',
		'retry_interval' => 'int',
		'check_period' => 'select',
		'obsess' => 'bool',
		'check_freshness' => 'bool',
		'freshness_threshold' => 'int',
		'active_checks_enabled' => 'bool',
		'passive_checks_enabled' => 'bool',
		'event_handler' => 'select',
		'event_handler_args' => 'string',
		'event_handler_enabled' => 'bool',
		'low_flap_threshold' => 'int',
		'high_flap_threshold' => 'int',
		'flap_detection_enabled' => 'bool',
		'flap_detection_options' => 'checkbox',
		'process_perf_data' => 'bool',
		'retain_status_information' => 'bool',
		'retain_nonstatus_information' => 'bool',
		'notification_interval' => 'int',
		'first_notification_delay' => 'int',
		'notification_period' => 'select',
		'notification_options' => 'checkbox',
		'notifications_enabled' => 'bool',
		'stalking_options' => 'checkbox',
		'register' => 'bool',
		'file_id' => 'select'
		);

$Object_Properties['host']['widget'] = array('register' => 'hidden');

/* again, adding hosts is a bit warped, so this doesn't get printed
 * with other objects of its kind(as other things tend to be) */
$Object_Properties['host']['default'] =
  array('template' => 'default-host-template',
		'host_name' => '',
		'alias' => '',
		'address' => '',
		'check_command' => 'check-host-alive',
		'parents' => '',
		'file_id' => 'etc/hosts.cfg',
		'register' => 1,
		);

$Object_Properties['host']['required'] =
  array('host_name',
		'template',
		'max_check_attempts',
		'notification_interval',
		'notification_period',
		'notification_options',
		'file_id');

$Object_Properties['host']['special'] =
  array('stalking_options' => array('d' => 'Down',
									'u' => 'Unreachable',
									'o' => 'Up',
									'n' => false),
		'notification_options' => array('d' => 'Down',
										'u' => 'Unreachable',
										'r' => 'Recovery',
										'f' => 'Flapping start and stop',
										's' => 'Scheduled downtime start and stop',
										'n' => false),
		'template' => 'host_template',
		'file_id' => 'FILE',
		'parents' => 'host',
		'children' => 'host',
		'check_command' => 'command',
		'event_handler' => 'command',
		'check_period' => 'timeperiod',
		'notification_period' => 'timeperiod',
		'hostgroups' => 'hostgroup',
		'contacts' => 'contact',
		'contact_groups' => 'contactgroup',
		);
$Object_Properties['host']['special']['flap_detection_options'] =
	$Object_Properties['host']['special']['stalking_options'];
$Object_Properties['host']['deprecated_names']['obsess_over_host'] = 'obsess';
/*****************************************************************************
 *            Data definitions for the 'hostgroup' object type               *
 *****************************************************************************/
$Object_Properties['hostgroup']['topic'] = 'hostgroup_name';
$Object_Properties['hostgroup']['var_index'] =
  array('hostgroup_name' => 'string',
		'alias' => 'string',
		'members' => 'mselect',
		'hostgroup_members' => 'mselect',
		'notes' => 'string',
		'notes_url' => 'notesurl',
		'action_url' => 'string',
		'file_id' => 'select',
		'register' => 'bool'
		);

$Object_Properties['hostgroup']['widget'] = array('register' => 'hidden');

$Object_Properties['hostgroup']['required'] =
  array('hostgroup_name', 'file_id');

$Object_Properties['hostgroup']['default'] =
  array('hostgroup_name' => '',
		'alias' => '',
		'members' => '',
		'notes' => '',
		'notes_url' => '',
		'action_url' => '',
		'register' => 1,
		'file_id' => 'etc/hostgroups.cfg'
		 );

$Object_Properties['hostgroup']['special'] =
  array('file_id' => 'FILE', 'members' => 'host', 'hostgroup_members' => 'hostgroup');

/*****************************************************************************
 *              Data definitions for the 'service' object type               *
 *****************************************************************************/
$Object_Properties['service']['topic'] = 'service_description';
$Object_Properties['service']['var_index'] =
  array('template' => 'select',
		'host_name' => 'select',
		'hostgroup_name' => 'select',
		'service_description' => 'string',
		'display_name' => 'string',
		'is_volatile' => 'bool',
		'check_command' => 'select',
		'check_command_args' => 'string',
		'servicegroups' => 'mselect',
		'max_check_attempts' => 'int',
		'check_interval' => 'int',
		'retry_interval' => 'int',
		'active_checks_enabled' => 'bool',
		'passive_checks_enabled' => 'bool',
		'check_period' => 'select',
		'parallelize_check' => 'bool',
		'obsess' => 'bool',
		'check_freshness' => 'bool',
		'freshness_threshold' => 'int',
		'event_handler' => 'select',
		'event_handler_args' => 'string',
		'event_handler_enabled' => 'bool',
		'low_flap_threshold' => 'int',
		'high_flap_threshold' => 'int',
		'flap_detection_enabled' => 'bool',
		'flap_detection_options' => 'checkbox',
		'process_perf_data' => 'bool',
		'retain_status_information' => 'bool',
		'retain_nonstatus_information' => 'bool',
		'notification_interval' => 'int',
		'first_notification_delay' => 'int',
		'notification_period' => 'select',
		'notification_options' => 'checkbox',
		'notifications_enabled' => 'bool',
		'contacts' => 'mselect',
		'contact_groups' => 'mselect',
		'stalking_options' => 'checkbox',
		'register' => 'bool',
		'file_id' => 'select',
		);

$Object_Properties['service']['widget'] = array(
	'host_name' => 'hidden',
	'hostgroup_name' => 'hidden',
	'register' => 'hidden',
);

$Object_Properties['service']['required'] =
  array('template',
		'host_name',
		'service_description',
		'check_command',
		'max_check_attempts',
		'check_interval',
		'retry_interval',
		'check_period',
		'notification_interval',
		'notification_period',
		'notification_options',
		'file_id'
		);

$Object_Properties['service']['req_alt'] =
  array('host_name' => 'hostgroup_name');

/* default values for new service objects are fetched from the first
 * configured object that isn't a template(to allow for 'strange'
 * configurations).
 * The values are then nulled out */
$Object_Properties['service']['default'] =
  array('template' => 'default-service',
		'host_name' => '',
		'service_description' => '',
		'check_command' => '',
		'check_command_args' => '',
		'contact_groups' => '',
		'register' => 1,
		'file_id' => 'etc/services.cfg');

$Object_Properties['service']['special'] =
  array('template' => 'service_template',
		'file_id' => 'FILE',
		'notification_period' => 'timeperiod',
		'check_period' => 'timeperiod',
		'check_command' => 'command',
		'event_handler' => 'command',
		'contacts' => 'contact',
		'contact_groups' => 'contactgroup',
		'servicegroups' => 'servicegroup',
		'host_name' => 'host',
		'hostgroup_name' => 'hostgroup'
		);

$Object_Properties['service']['special']['stalking_options'] =
  array('c' => 'Critical',
		'w' => 'Warning',
		'u' => 'Unknown',
		'o' => 'OK',
		'n' => false,
		);

$Object_Properties['service']['special']['notification_options'] =
  array('c' => 'Critical',
		'w' => 'Warning',
		'u' => 'Unknown',
		'r' => 'Recovery',
		'f' => 'Flapping start and stop',
		's' => 'Scheduled downtime start and stop',
		'n' => false);

$Object_Properties['service']['special']['flap_detection_options'] =
	$Object_Properties['service']['special']['stalking_options'];

$Object_Properties['service']['deprecated_names']['obsess_over_service'] = 'obsess';

/*****************************************************************************
 *              Data definitions for the 'contact' object type               *
 *****************************************************************************/
$Object_Properties['contact']['topic'] = 'contact_name';
$Object_Properties['contact']['var_index'] =
  array('template' => 'select',
		'contact_name' => 'string',
		'alias' => 'string',
		'host_notifications_enabled' => 'bool',
		'service_notifications_enabled' => 'bool',
		'can_submit_commands' => 'bool',
		'retain_status_information' => 'bool',
		'retain_nonstatus_information' => 'bool',
		'host_notification_period' => 'select',
		'service_notification_period' => 'select',
		'host_notification_options' => 'checkbox',
		'service_notification_options' => 'checkbox',
		'host_notification_cmds' => 'select',
		'host_notification_cmds_args' => 'string',
		'service_notification_cmds' => 'select',
		'service_notification_cmds_args' => 'string',
		'contactgroups' => 'mselect',
		'register' => 'bool',
		'email' => 'string',
		'pager' => 'string',
		'address1' => 'string',
		'address2' => 'string',
		'address3' => 'string',
		'address4' => 'string',
		'address5' => 'string',
		'address6' => 'string',
		'file_id' => 'select',
		);

$Object_Properties['contact']['widget'] = array('register' => 'hidden');

$Object_Properties['contact']['required'] =
  array('contact_name',
		'alias',
		'host_notification_period',
		'service_notification_period',
		'host_notification_options',
		'service_notification_options',
		'file_id'
		);

$Object_Properties['contact']['default'] =
  array('template' => 'default-contact',
		'contact_name' => '',
		'alias' => '',
		'service_notification_period' => '24x7',
		'host_notification_period' => '24x7',
		'service_notification_options' => 'c,w,r',
		'host_notification_options' => 'd,r',
		'host_notifications_enabled' => 1,
		'service_notifications_enabled' => 1,
		'can_submit_commands' => 0,
		'retain_status_information' => 1,
		'retain_nonstatus_information' => 1,
		'service_notification_cmds' => 'service-notify',
		'host_notification_cmds' => 'host-notify',
		'contactgroups' => '',
		'email' => '',
		'pager' => '',
		'address1' => '',
		'address2' => '',
		'address3' => '',
		'address4' => '',
		'address5' => '',
		'address6' => '',
		'register' => 1,
		'file_id' => 'etc/contacts.cfg',
		 );

$Object_Properties['contact']['special'] =
  array('host_notification_options' => array('d' => 'Down',
											 'u' => 'Unreachable',
 											 'r' => 'Recovery',
											 'f' => 'Flapping start and stop',
											 's' => 'Scheduled downtime start and stop',
 											 'n' => false),
		'service_notification_options' => array('c' => 'Critical',
												'w' => 'Warning',
												'u' => 'Unknown',
												'r' => 'Recovery',
												'f' => 'Flapping start and stop',
												's' => 'Scheduled downtime start and stop',
												'n' => false),
		'file_id' => 'FILE',
		'template' => 'contact_template',
		'host_notification_period' => 'timeperiod',
		'service_notification_period' => 'timeperiod',
		'host_notification_cmds' => 'command',
		'service_notification_cmds' => 'command',
		'contactgroups' => 'contactgroup'
		);
$Object_Properties['contact']['label'] = array();
$Object_Properties['contact']['deprecated_names'] = array();

/*****************************************************************************
 *           Data definitions for the 'contactgroup' object type             *
 *****************************************************************************/
$Object_Properties['contactgroup']['topic'] = 'contactgroup_name';
$Object_Properties['contactgroup']['var_index'] =
  array('contactgroup_name' => 'string',
		'alias' => 'string',
		'members' => 'mselect',
		'contactgroup_members' => 'mselect',
		'file_id' => 'select',
		'register' => 'bool'
	);

$Object_Properties['contactgroup']['widget'] = array('register' => 'hidden');

$Object_Properties['contactgroup']['required'] =
  array('contactgroup_name',
		'alias',
		'file_id');

$Object_Properties['contactgroup']['default'] =
  array('contactgroup_name' => '',
		'alias' => '',
		'members' => '',
		'register' => 1,
		'file_id' => 'etc/contactgroups.cfg');

$Object_Properties['contactgroup']['special'] =
  array('members' => 'contact',
		'contactgroup_members' => 'contactgroup',
		'file_id' => 'FILE');

/*****************************************************************************
 *             Data definitions for the 'timeperiod' object type             *
 *****************************************************************************/
/* Timeperiods are always expanded. Thus, these definitions differ a bit
 * from the 'usual' style. */
$Object_Properties['timeperiod']['topic'] = 'timeperiod_name';
$Object_Properties['timeperiod']['var_index'] =
  array('timeperiod_name' => 'string',
		'alias' => 'string',
		'exclude' => 'mselect',
 		'sunday' => 'string',
		'monday' => 'string',
		'tuesday' => 'string',
		'wednesday' => 'string',
		'thursday' => 'string',
		'friday' => 'string',
		'saturday' => 'string',
		'file_id' => 'select',
		'register' => 'bool'
		);

$Object_Properties['timeperiod']['widget'] = array('register' => 'hidden');

$Object_Properties['timeperiod']['required'] =
  array('timeperiod_name',
		'alias',
		'file_id');

$Object_Properties['timeperiod']['default'] =
  array('timeperiod_name' => '',
		'alias' => '',
		'exclude' => '',
 		'sunday' => '',
		'monday' => '',
		'tuesday' => '',
		'wednesday' => '',
		'thursday' => '',
		'friday' => '',
		'saturday' => '',
		'register' => 1,
		'file_id' => 'etc/timeperiods.cfg'
		);

$Object_Properties['timeperiod']['special'] =
	array('file_id' => 'FILE', 'exclude' => 'timeperiod');

/*****************************************************************************
 *             Data definitions for the 'command' object type                *
 *****************************************************************************/
$Object_Properties['command']['topic'] = 'command_name';

$Object_Properties['command']['var_index'] =
  array('command_name' => 'string',
		'command_line' => 'string',
		'file_id' => 'select',
		'register' => 'bool'
	);

$Object_Properties['command']['widget'] = array('register' => 'hidden');

$Object_Properties['command']['required'] =
  array('command_name',
		'command_line',
		'file_id');

$Object_Properties['command']['default'] =
  array('command_name' => '',
		'command_line' => '',
		'register' => 1,
		'file_id' => 'etc/checkcommands.cfg'
		);
$Object_Properties['command']['special']['file_id'] = 'FILE';

/*****************************************************************************
 *          Data definitions for the 'hostextinfo' object type               *
 *****************************************************************************/
$Object_Properties['hostextinfo']['topic'] = null;
$Object_Properties['hostextinfo']['var_index'] =
  array('host_name' => 'select',
		'icon_image' => 'image',
		'icon_image_alt' => 'string',
		'statusmap_image' => 'image',
		'notes' => 'string',
		'action_url' => 'actionurl',
		'notes_url' => 'notesurl',
		'2d_coords' => 'string',
		'file_id' => 'select',
		'register' => 'bool',
		);

$Object_Properties['hostextinfo']['widget'] = array(
	'host_name' => 'hidden',
	'2d_coords' => 'hidden',
	'register' => 'hidden',
);

$Object_Properties['hostextinfo']['required'] =
  array('host_name', 'file_id');

$Object_Properties['hostextinfo']['special'] =
  array('host_name' => 'host',
		'icon_image' => $IMAGES,
		'statusmap_image' => $IMAGES,
		'file_id' => 'FILE'
		);

foreach ($Object_Properties['hostextinfo']['special'] as $k => $v)
	if (!isset($Object_Properties['host']['var_index'][$k]))
		$Object_Properties['host']['special'][$k] = $v;

foreach ($Object_Properties['hostextinfo']['var_index'] as $k => $v)
	if (!isset($Object_Properties['host']['var_index'][$k]))
		$Object_Properties['host']['var_index'][$k] = $v;

$Object_Properties['hostextinfo']['default'] =
  array('host_name' => '',
		'icon_image' => '',
		'icon_image_alt' => '',
		'statusmap_image' => '',
		'notes' => '',
		'action_url' => '',
		'notes_url' => '',
		'2d_coords' => '',
		'register' => 1,
		'file_id' => 'etc/hostextinfo.cfg'
		);

/* dependency and escalations-stuff is only here for future support */
/*****************************************************************************
 *       Data definitions for the 'servicedependency' object type            *
 *****************************************************************************/
$Object_Properties['servicedependency']['topic'] = null;
$Object_Properties['servicedependency']['var_index'] =
  array('service' => 'select',
		'dependent_service' => 'select',
		'dependency_period' => 'select',
		'execution_failure_criteria' => 'checkbox',
		'notification_failure_criteria' => 'checkbox',
		'inherits_parent' => 'bool',
		'file_id' => 'select',
		'register' => 'bool'
		);

$Object_Properties['servicedependency']['widget'] = array(
	'dependent_service' => 'hidden',
	'register' => 'hidden',
);

$Object_Properties['servicedependency']['required'] =
  array('service',
		'dependent_service',
		'file_id'
		);

$Object_Properties['servicedependency']['default'] =
  array('service' => '',
		'dependent_service' => '',
		'execution_failure_criteria' => 'c,w',
		'notification_failure_criteria' => 'c,w',
		'inherits_parent' => '1',
		'register' => 1,
		'file_id' => 'etc/servicedependencies.cfg'
		);

$Object_Properties['servicedependency']['special'] =
  array('service' => 'service',
		'dependent_service' => 'service',
		'execution_failure_criteria' => array('c' => 'Critical',
											  'w' => 'Warning',
											  'u' => 'Unknown',
											  'o' => 'OK',
											  'n' => false),
		'notification_failure_criteria' => array('c' => 'Critical',
												 'w' => 'Warning',
												 'u' => 'Unknown',
												 'o' => 'OK',
												 'n' => false),
		'dependency_period' => 'timeperiod',
		'file_id' => 'FILE'
		);

/*****************************************************************************
 *       Data definitions for the 'serviceescalations' object type           *
 *****************************************************************************/
$Object_Properties['serviceescalation']['topic'] = null;
$Object_Properties['serviceescalation']['var_index'] =
  array('service' => 'select',
		'contacts' => 'mselect',
		'contact_groups' => 'mselect',
		'first_notification' => 'int',
		'last_notification' => 'int',
		'notification_interval' => 'int',
		'escalation_period' => 'select',
		'escalation_options' => 'checkbox',
		'register' => 'bool',
		'file_id' => 'select',
		);

$Object_Properties['serviceescalation']['widget'] = array(
	'service' => 'hidden',
	'register' => 'hidden',
);

$Object_Properties['serviceescalation']['default'] =
  array('service' => '',
		'contact_groups' => '',
		'contacts' => '',
		'first_notification' => '',
		'last_notification' => '',
		'notification_interval' => '',
		'escalation_period' => '',
		'escalation_options' => 'w,c',
		'register' => 1,
		'file_id' => 'etc/serviceescalations.cfg'
		);

$Object_Properties['serviceescalation']['required'] =
	array('service', 'file_id');

$Object_Properties['serviceescalation']['special'] =
  array('service' => 'service',
		'contacts' => 'contact',
		'contact_groups' => 'contactgroup',
		'escalation_period' => 'timeperiod',
		'escalation_options' => array('c' => 'Critical',
									  'w' => 'Warning',
									  'u' => 'Unknown',
									  'r' => 'Recovery',
									  'n' => false
									  ),
		'file_id' => 'FILE'
		);

/*****************************************************************************
 *          Data definitions for the 'servicegroup' object type              *
 *****************************************************************************/
$Object_Properties['servicegroup']['topic'] = 'servicegroup_name';
$Object_Properties['servicegroup']['var_index'] =
  array('servicegroup_name' => 'string',
		'alias' => 'string',
		'members' => 'mselect',
		'servicegroup_members' => 'mselect',
		'notes' => 'string',
		'notes_url' => 'notesurl',
		'action_url' => 'string',
		'file_id' => 'select',
		'register' => 'bool'
		);

$Object_Properties['servicegroup']['widget'] = array('register' => 'hidden');

$Object_Properties['servicegroup']['required'] =
  array('servicegroup_name', 'file_id');

$Object_Properties['servicegroup']['default'] =
  array('servicegroup_name' => '',
		'alias' => '',
		'members' => '',
		'register' => 1,
		'file_id' => 'etc/servicegroups.cfg'
		);

$Object_Properties['servicegroup']['special'] =
  array('members' => 'service',
		'servicegroup_members' => 'servicegroup',
		'file_id' => 'FILE');

/*****************************************************************************
 *         Data definitions for the 'serviceextinfo' object type             *
 *****************************************************************************/
$Object_Properties['serviceextinfo']['topic'] = null;
$Object_Properties['serviceextinfo']['var_index'] =
  array('service' => 'select',
		'notes' => 'string',
		'notes_url' => 'notesurl',
		'action_url' => 'actionurl',
		'icon_image' => 'image',
		'icon_image_alt' => 'string',
		'file_id' => 'select',
		'register' => 'bool',
		);

$Object_Properties['serviceextinfo']['widget'] = array(
	'service' => 'hidden',
	'register' => 'hidden',
);

$Object_Properties['serviceextinfo']['default'] =
  array('notes' => '',
		'notes_url' => '',
		'action_url' => '',
		'icon_image' => '',
		'icon_image_alt' => '',
		'register' => 1,
		'file_id' => 'etc/serviceextinfo.cfg',
		);

$Object_Properties['serviceextinfo']['special'] =
  array('service' => 'service',
		'icon_image' => $IMAGES,
		'file_id' => 'FILE'
		);

foreach ($Object_Properties['serviceextinfo']['special'] as $k => $v) {
	if ($k === 'service')
		continue;
	if (!isset($Object_Properties['service']['var_index'][$k]))
		$Object_Properties['service']['special'][$k] = $v;
}

foreach ($Object_Properties['serviceextinfo']['var_index'] as $k => $v) {
	if ($k === 'service')
		continue;
	if (!isset($Object_Properties['service']['var_index'][$k]))
		$Object_Properties['service']['var_index'][$k] = $v;
}

$Object_Properties['serviceextinfo']['required'] =
  array('service', 'file_id');

/*****************************************************************************
 *         Data definitions for the 'hostdependency' object type             *
 *****************************************************************************/
$Object_Properties['hostdependency']['topic'] = null;
$Object_Properties['hostdependency']['var_index'] =
  array('host_name' => 'select',
		'hostgroup_name' => 'select',
		'dependent_host_name' => 'select',
		'dependent_hostgroup_name' => 'select',
		'dependency_period' => 'select',
		'execution_failure_criteria' => 'checkbox',
		'notification_failure_criteria' => 'checkbox',
		'inherits_parent' => 'bool',
		'file_id' => 'select',
		'register' => 'bool'
		);

$Object_Properties['hostdependency']['widget'] = array(
	'dependent_host_name' => 'hidden',
	'dependent_hostgroup_name' => 'hidden',
	'register' => 'hidden'
);

$Object_Properties['hostdependency']['default'] =
  array('dependent_host_name' => '',
		'host_name' => '',
		'execution_failure_criteria' => 'n',
		'notification_failure_criteria' => 'd,u',
		'inherits_parent' => '1',
		'register' => 1,
		'file_id' => 'etc/hostdependencies.cfg'
		);

$Object_Properties['hostdependency']['required'] =
  array('host_name',
		'dependent_host_name',
		'notification_failure_criteria',
		'file_id'
		);

$Object_Properties['hostdependency']['req_alt'] =
  array('host_name' => 'hostgroup_name',
		'dependent_host_name' => 'dependent_hostgroup_name');

$Object_Properties['hostdependency']['special'] =
  array('dependent_host_name' => 'host',
		'host_name' => 'host',
		'hostgroup_name' => 'hostgroup',
		'dependent_hostgroup_name' => 'hostgroup',
		'notification_failure_criteria' => array('d' => 'Down',
												 'u' => 'Unreachable',
												 'o' => 'Up',
												 'n' => false),
		'execution_failure_criteria' => array('d' => 'Down',
											  'u' => 'Unreachable',
											  'o' => 'Up',
											  'n' => false),
		'dependency_period' => 'timeperiod',
		'file_id' => 'FILE'
		);

/*****************************************************************************
 *         Data definitions for the 'hostescalation' object type             *
 *****************************************************************************/
$Object_Properties['hostescalation']['topic'] = null;
$Object_Properties['hostescalation']['var_index'] =
  array('host_name' => 'select',
		'hostgroup_name' => 'select',
		'contacts' => 'mselect',
		'contact_groups' => 'mselect',
		'first_notification' => 'int',
		'last_notification' => 'int',
		'notification_interval' => 'int',
		'escalation_period' => 'select',
		'escalation_options' => 'checkbox',
		'register' => 'bool',
		'file_id' => 'select'
		);

$Object_Properties['hostescalation']['widget'] = array(
	'host_name' => 'hidden',
	'hostgroup_name' => 'hidden',
	'register' => 'hidden',
);

$Object_Properties['hostescalation']['default'] =
  array('host_name' => '',
		'contacts' => '',
		'contact_groups' => '',
		'first_notification' => '',
		'last_notification' => '',
		'notification_interval' => '',
		'escalation_period' => '',
		'escalation_options' => 'd',
		'register' => 1,
		'file_id' => 'etc/hostescalations.cfg'
		);

$Object_Properties['hostescalation']['required'] =
  array('host_name',
		'first_notification',
		'last_notification',
		'notification_interval',
		'file_id'
		);

$Object_Properties['hostescalation']['req_alt'] =
  array('host_name' => 'hostgroup_name');

$Object_Properties['hostescalation']['special'] =
  array('host_name' => 'host',
		'hostgroup_name' => 'hostgroup',
		'contacts' => 'contact',
		'contact_groups' => 'contactgroup',
		'escalation_period' => 'timeperiod',
		'escalation_options' => array('d' => 'Down',
									  'u' => 'Unreachable',
									  'r' => 'Recovery',
									  'n' => false),
		'file_id' => 'FILE'
		);

/*****************************************************************************
 *             Data definitions for the 'new_host' object type               *
 *****************************************************************************/

/* this is a hack. it's only here so that we can use the 'pretty_print'
 * routine to get info on new hosts and the validate_object function to
 * verify that everything's alright */
$Object_Properties['new_host']['topic'] = 'host_name';
$Object_Properties['new_host']['var_index'] =
  array('Add this host?' => 'bool',
		'template' => 'select',
		'host_name' => 'string',
		'alias' => 'string',
		'address' => 'string',
		'contacts' => 'mselect',
		'contact_groups' => 'mselect',
		'hostgroups' => 'mselect',
		'parents' => 'mselect',
		'display_name' => 'string',
//		'SNMP Community' => 'string',
		'Autodetect Services' => 'checkbox',
		'Service Checks' => 'checkbox',
		'action_url' => 'actionurl',
		'Host logo' => 'image',
		'file_id' => 'select'
		);

$Object_Properties['new_host']['default'] =
  array('Add this host?' => '1',
		'template' => 'default-host-template',
		'host_name' => '',
		'hostgroups' => '',
		'alias' => '',
		'address' => '',
		'parents' => '',
		'contact_groups' => '', // unset contactgroup (fix later)
		'Autodetect Services' => 'net',
		'Service Checks' => '',
//		'SNMP Community' => '',
		'action_url' => '',
		'Host logo' => '',
		'file_id' => 'etc/hosts.cfg'
		);

$Object_Properties['new_host']['required'] =
  array('host_name', 'template', 'file_id');

$Object_Properties['new_host']['special'] =
  array('template' => 'host_template',
		'file_id' => 'FILE',
		'hostgroups' => 'hostgroup',
		'parents' => 'host',
		'Autodetect Services' => array('net' => 'Autodetect Network Services(PING, SMTP, agents, etc)'),
		'Service Checks' => array('nrpe' => 'Add UNIX Client Services(NRPE)',
							'nsclient' => 'Add Windows Client Services(NSClient)',
							'sensatronics-e4' => 'Add Sensatronics E4',
							'sensatronics-em1' => 'Add Sensatronics EM1',
							'netware' => 'Add NetWare Client Services(NWStat)',
							'wmi' => 'Add WMI'
//							'snmp' => 'Autodetect SNMP capability'
							),
		'Host logo' => $IMAGES,
		'contact_groups' => 'contactgroup',
		'contacts' => 'contact',
		);
/*****************************************************************************
 *          Data definitions for the 'new_service' object type               *
 *****************************************************************************/

/* like new_host, this is a hack. It's only here to allow us to use the
 * pretty print function from probe.php */
$Object_Properties['new_service']['topic'] = false;
$Object_Properties['new_service']['var_index'] =
  $Object_Properties['service']['var_index'];
$Object_Properties['new_service']['special'] =
  $Object_Properties['service']['special'];
$Object_Properties['new_service']['required'] = array('template', 'file_id');
$Object_Properties['new_service']['default'] =
  array('template' => 'default-service',
		'file_id' => 'etc/services.cfg'
		);

/*****************************************************************************
 *          Data definitions for the 'FILE' object type               *
 *****************************************************************************/
# This is a simple hack to be able to use the get_object_by_name() method
# for FILE objects from API
$Object_Properties['FILE']['var_index'] = array('file_name' => 'string');
$Object_Properties['FILE']['default'] = array('file_name' => '');
$Object_Properties['FILE']['special'] = array('file_id' => 'FILE');
$Object_Properties['FILE']['required'] = array('file_name');
$Object_Properties['FILE']['topic'] = 'file_name';
