<?php
/*
 * Copyright(C) 2003-2005, 2007 op5 AB
 * All rights reserved.
 *
 */

require_once('include/import.inc.php');
require_once('include/webconfig.inc.php');
require_once('include/check_mk.inc.php');
require_once('include/object_properties.inc.php');
require_once('include/objlock.inc.php');
require_once('classes/common_gui.php');
require_once('op5/log.php');

function generate_link($label, $page_link, $icon=false) {
	if ($icon == false)
		$icon = str_replace(' ','-',strtolower($label));
	return '<li><img src="images/icons/'.$icon.'.png" alt="" style="display: inline" /> <a href="'.$page_link.'">'.$label.'</a></li>';
}

$access = get_nacoma_access('configuration_information');
if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
	$page->html_start("Access denied!");
	echo "<p class=\"note\">It appears as though you aren't authorized to access the configuration interface.</p>
			<p>Note that at least one of your groups must be authorized for both <strong>'configuration_information'</strong>
			and <strong>'system_commands'</strong> to be able change any configuration.</p>";
	$page->html_end();
} // html_end exists

$page->html_start('Configure');

die_if_locked_hard();

$rights = get_rights();
# import or not?
if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'Import') {
	$rights = get_rights();
	ob_start();
	if ((isset($rights['configuration_all'])) && isset($_REQUEST['force'])) {
		$page->run_import('Import forced');
	} else {
		revert_users_changes();
		if (isset($rights['configuration_all']))
			$extra_content = '<p>You might want to do a <a href="index.php?action=Import&force=force">complete reimport</a>?</p>';
	}
	$content = ob_get_clean();
	if(isset($extra_content)) {
		$content .= $extra_content;
		unset($extra_content); // wow, we're in the global scope
	}
	if($content) {
		echo '<div class="note">'.$content.'</div>';
		unset($content); // wow, we're in the global scope
	}
	check_mk_config_cleanup();
}

$page->import_if_changes();

echo '<p>This is the op5 Monitor Configuration tool - the place where you add new hosts,
change notification settings, handle contacts and so on. Basically just about
every configuration related setting is managed from this tool.</p>';

echo "<p><strong>To get started, select the prefered action below.</strong></p></div>";

// this must be done AFTER importing.
get_object_index_table('host');

$Menu_Item = array(
	'hostsvc' => array(), // Core host and service related objects
	'contact' => array(), // Stuff involved with contacts
	'template' => array(), // Well, just templates
	'auth' => array(), // All auth related pages
	'admin' => array(), // Nagios related stuff that's not checks, ie commands and stuff
	'graph' => array(), // PNP related stuff
);

$menu_headers = array(
	'hostsvc' => 'Hosts and Services',
	'contact' => 'Notification Contacts',
	'template' => 'Templates',
	'auth' => 'Permissions',
	'admin' => 'Core Configuration',
	'graph' => 'Graphs',
);

$hosts = get_objects_for_user('host');
if (!empty($hosts)) {
	$Menu_Item['hostsvc'][] = '<li class="wide"><form name="host_form" id="host_form" action="edit.php" method="get">'.
		'<input type="hidden" name="obj_type" value="host" />'.
		'<img src="images/icons/host.png" alt="" />Edit host '.
		$page->make_filtered_selection($hosts, '', 'obj_id').
		'<input type="hidden" value="Go" name="action">'.
		'<input class="object-submit" type="submit" onclick="loopElements(this.form);" title="Go" value="Go" name="action_dummy">'.
		'</form></li>';
}

if(object_class::can_user_write('host', 'new') && object_class::can_user_write('service', 'new')) {
	$Menu_Item['hostsvc'][] = generate_link('Add new hosts', 'host_wizard.php', 'new-host');
	$Menu_Item['hostsvc'][] = generate_link('Network autoscan', 'autoscan.php');
}

if (in_array(NACOMA_ACCESS_VIEW_ALL, get_nacoma_access('host_template')))
	$Menu_Item['template'][] = generate_link('Host templates', 'edit.php?obj_type=host_template', 'templates');

if (in_array(NACOMA_ACCESS_VIEW_ALL, get_nacoma_access('service_template')))
	$Menu_Item['template'][] = generate_link('Service templates', 'edit.php?obj_type=service_template', 'templates');

if (in_array(NACOMA_ACCESS_VIEW_ALL, get_nacoma_access('host_template')))
	$Menu_Item['template'][] = generate_link('Contact templates', 'edit.php?obj_type=contact_template', 'templates');

$access = get_nacoma_access('hostgroup');
if (in_array(NACOMA_ACCESS_VIEW_ALL, $access) || in_array(NACOMA_ACCESS_VIEW_CONTACT, $access))
	$Menu_Item['hostsvc'][] = generate_link('Host groups', 'edit.php?obj_type=hostgroup');
$access = get_nacoma_access('management_pack');
if (in_array(NACOMA_ACCESS_VIEW_ALL, $access) || in_array(NACOMA_ACCESS_VIEW_CONTACT, $access)) {
	$Menu_Item['hostsvc'][] = generate_link('Management packs', 'edit.php?obj_type=management_pack');
	if (in_array(NACOMA_ACCESS_EDIT_ALL, $access)) {
		$Menu_Item['admin'][] = generate_link('Manage Management packs', 'mgmtpackimport.php');
	}
}
$access = get_nacoma_access('servicegroup');
if (in_array(NACOMA_ACCESS_VIEW_ALL, $access) || in_array(NACOMA_ACCESS_VIEW_CONTACT, $access))
	$Menu_Item['hostsvc'][] = generate_link('Service groups', 'edit.php?obj_type=servicegroup');
$access = get_nacoma_access('contact');
if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
	$Menu_Item['contact'][] = generate_link('Contacts', 'edit.php?obj_type=contact');
$access = get_nacoma_access('contactgroup');
if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
	$Menu_Item['contact'][] = generate_link('Contact groups', 'edit.php?obj_type=contactgroup');
$access = get_nacoma_access('command');
if (in_array(NACOMA_ACCESS_VIEW_ALL, $access)) {
	$Menu_Item['admin'][] = generate_link('Commands', 'edit.php?obj_type=command');
	$Menu_Item['admin'][] = generate_link('Check command import', 'metadata.php');
}
$Menu_Item['admin'][] = generate_link('Plugin search', 'plugin_search.php');
$access = get_nacoma_access('timeperiod');
if (in_array(NACOMA_ACCESS_VIEW_ALL, $access))
	$Menu_Item['admin'][] = generate_link('Time periods', 'edit.php?obj_type=timeperiod');

$access = get_nacoma_access('nagvis');
if (in_array(NACOMA_ACCESS_ADD_DELETE, $access)) {
	$Menu_Item['auth'][] = generate_link('Nagvis permissions', 'nagvisls.php');
}


$access = get_nacoma_access('access_rights');
if (in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
	// Local Users are only relevant if a local auth driver is configured.
	$userdrivers = $op5_auth->getAuthModulesPerDriver( 'Default' );
	if( false !== $userdrivers ) {
		$Menu_Item['auth'][] = generate_link('Local users', 'edit_special.php/access');
	}
	$Menu_Item['auth'][] = generate_link('Auth modules', 'authconfig.php');
	$Menu_Item['auth'][] = generate_link('Assign group rights', 'edit_special.php/group');
}


$access = get_nacoma_access('pnp');
if (in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
	$Menu_Item['graph'][] = generate_link('Combined graphs', 'edit.php?obj_type=combined_graph');
	$Menu_Item['graph'][] = generate_link('Graph collections', 'edit.php?obj_type=graph_collection');
	$Menu_Item['graph'][] = generate_link('Graph templates', 'edit.php?obj_type=graph_template');
}

foreach($Menu_Item as $category => $items) {
	if (empty($items))
		continue;
	echo '<h2 class="confighdr">'.$menu_headers[$category].'</h2>';
	echo "<ul class=\"config\">\n";
	foreach ($items as $item) {
		echo "$item\n";
	}
	echo "</ul>\n";
}

?>
<p style="clear:both; padding-top: 30px"><strong>Tip:</strong> If you need help you can always click on the
<img src="images/icons/square-help.png" alt="" style="margin-bottom: -3px" />
in the upper right corner, you can also click on the
<img src="images/icons/shield-help.png" alt="" style="margin-bottom: -2px" /> for each configurable variable.</p>

<?php
$page->html_end();
