<?php
require_once('object_class.php');

/**
 * A contact definition is used to identify someone who should
 * be contacted in the event of a problem on your network. This is
 * not always a user, which means not all contacts can log into the
 * Monitor GUI.
 */
class contact_class extends object_class
{
	/** object type */
	static $type = 'contact';

	/** Nagios' default settings */
	var $nagios_defaults = array(
		'host_notification_options' => array('n' => ''),
		'service_notification_options' => array('n' => ''),
		'host_notifications_enabled' => 1,
		'service_notifications_enabled' => 1,
		'can_submit_commands' => 0,
		'retain_status_information' => 1,
		'retain_nonstatus_information' => 1
	);

	public function get_properties()
	{
		parent::get_properties();
		$this->properties['var_index']['enable_access'] = 'enable_access';
		$this->properties['default']['enable_access'] = false;
		$user = new user_class();
		foreach($user->get_properties() as $property => $value) {
			if ($property === 'topic')
				continue;
			if(isset($value['username'])) {
				unset($value['username']);
			}

			$this->properties[$property] = array_merge($this->properties[$property], $value);
		}
		return $this->properties;
	}

	function save_object($user = true) {
		$real_props = $this->properties;
		$real_obj = $this->obj;
		$enable_access = isset($this->obj['enable_access']) && $this->obj['enable_access'];
		if (isset($this->obj['enable_access']))
			unset($this->obj['enable_access']);

		// remove the merged properties used for form generation
		$userpasswd = new user_class();
		foreach($userpasswd->get_properties() as $property => $value) {
			if ($property === 'topic')
				continue;
			foreach($value as $sub_property => $real_value) {
				unset($this->properties[$property][$sub_property]);
			}
		}

		if ($enable_access) {
			// ... and add the values the user gave us via the contact form
			foreach(array_keys($userpasswd->properties['var_index']) as $property) {
				if(!isset($this->obj[$property])) {
					// this if case is included because Username <> contact_name,
					// resolved outside foreach-loop
					continue;
				}
				$userpasswd->obj[$property] = $this->obj[$property];
				unset($this->obj[$property]);
			}
			if(isset($this->obj['contact_name']) && $this->obj['contact_name']) {
				$userpasswd->obj['username'] = $this->obj['contact_name'];
			}
		}
		else {
			foreach(array_keys($userpasswd->properties['var_index']) as $property) {
				if(isset($this->obj[$property])) {
					unset($this->obj[$property]);
				}
			}
			return parent::save_object($user);
		}

		if(!$this->validate_object()) {
			return false;
		}

		if(!$userpasswd->validate_object()) {
			$this->validation_error_list = $userpasswd->validation_error_list;
			return false;
		}

		if(!parent::save_object($user)) {
			return false;
		}

		$res = $userpasswd->save_object($user);

		$this->properties = $real_props;
		$this->obj = $real_obj;

		return $res;
	}
	
	/**
	 * Export with naming and values to map nagios struct names ond format
	 * 
	 * Useful for Test this x operation
	 */
	public function get_nagios_struct($prefix='') {
		$this->expand_object();
		$obj = $this->denormalize();
		return array(
				/* FIXME */
				);
	}
}
