<?php
require_once('object_class.php');

/**
 * Extinfo related methods.
 * hostextinfo and serviceextinfo share some functions, so
 * we put those here and let those two classes extend this
 * one.
 */
class extinfo_class extends object_class
{
	/**
	 * extinfo objects have a 1-to-1 relation with their
	 * master objects, so if we have the master_obj_id
	 * but not the id, we can still fetch the correct
	 * object
	 * @return object array
	 */
	function get_object()
	{
		# if we already have the id, or have neither id
		# or master_obj_id we take the default path
		if (($this->id === 'new' || !$this->id) && !$this->master_obj_id) {
			$this->obj = $this->properties['default'];
			$this->set_object_id('new');
			return $this->obj;
		}

		# since extinfo objects have no relations apart
		# from the one to their master, fetching the
		# object from database is as simple as finding
		# the exact entry
		$query = "SELECT * FROM $this->db_table WHERE ";
		if ($this->id && $this->id != 'new')
		  $query .= "id = $this->id";
		else
		  $query .= "$this->master_obj_identifier = $this->master_obj_id";

		$result = pdo_exec_query($query);
		$this->obj = pdo_fetch_object($result);
                unset($result);

		return $this->obj;
	}

	/**
	 * Validates that an object is correct prior to saving it to
	 * the database.
	 * We need to special case this to avoid convoluting
	 * object_class::validate_object() overly.
	 * Specifically, we need to rip out the images and re-insert them
	 * after the real validation is done.
	 * @return True if the object validates. False if it doesn't.
	 */
	function validate_object()
	{
		$image_vars = array('icon_image', 'statusmap_image');
		foreach ($image_vars as $k) {
			if (!isset($this->obj[$k]))
			  continue;
			$this->$k = $this->obj[$k];
			unset($this->obj[$k]);
		}

		$result = parent::validate_object();

		foreach ($image_vars as $k) {
			if (isset($this->$k)) {
				# Make sure images are saved with their name
				# rather than the index into the glob-list
				if (is_numeric($this->$k) && isset($this->properties['special']['icon_image'][$this->$k]))
					$this->$k = $this->properties['special']['icon_image'][$this->$k];

				$this->obj[$k] = $this->$k;
				unset($this->$k);
			}
		}

		return $result;
	}

	/**
	 * Set the master object id
	 * extinfo objects are unique in that their id can be
	 * determined uniquely by having the parent id, so try
	 * doing that when we get the master_object_id passed to us
	 */
	function set_master_object_id($id)
	{
		parent::set_master_object_id($id);

		# in case we're being cloned, $id will be false
		# and we must not try to use it below
		if (!$id)
			return;

		# only fetch the id and object if we don't already
		# have one. We could be just moving this object
		# to a new master (ie, cloning it)
		if ($this->id && $this->id !== 'new')
			return;

		$query = "SELECT id FROM " . $this->type . " WHERE " .
			$this->master_obj_identifier . " = " . pdo_quote_string($this->master_obj_id);
		$result = pdo_exec_query($query);
		$row = $result->fetch(PDO::FETCH_NUM);
		unset($result);
		if ($row) {
			$this->set_object_id($row[0]);
			$this->get_object();
		}
	}
}
