<?php
require_once('object_class.php');

/**
 * A host definition is used to define a physical server, workstation, device,
 * etc. that resides on your network.
 */
class host_class extends object_class
{
	/** The nagios object type we're working with */
	static $type = 'host';
	/** Default values that Nagios assumes if no values are set */
	var $nagios_defaults =
		array('stalking_options' => array('n' => ''),
		      'notification_options' => array('n' => ''),
		      'flap_detection_options' => array('o' => 1, 'd' => 1, 'u' => 1),
		      'obsess' => '1',
		      'check_freshness' => '0',
		      );
	var $slave_types = array(
		'hostextinfo',
		'hostescalation',
		'hostdependency',
		'service');

	function save_object($user=true)
	{
		$ret = parent::save_object($user);
		if (!$ret)
			return $ret;

		$dep_objs = $this->find_slave_objects('hostextinfo');
		if (isset($dep_objs)) {
			foreach ($dep_objs as $extinfo_id) {
				$extinfo = new hostextinfo_class($extinfo_id);
				$extinfo->banish_object();
			}
		}

		return $ret;
	}

	function validate_object() {
		if (!empty($this->obj['host_name']) && empty($this->obj['address']))
			$this->obj['address'] = $this->obj['host_name'];

		return parent::validate_object();
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = parent::get_cmd_bar_links();
		// instead of duplicating validation from parent, just check if the validation ended up allowing copying, and only then add clone
		if (isset($cmd_links['copy']))
			$cmd_links['clone'] = '<a title="Clone host" href="clone.php?obj_type=host&amp;obj_id='.$this->id.'">';
		return $cmd_links;
	}

	/**
	 * Export with naming and values to map nagios struct names ond format
	 *
	 * Useful for Test this x operation
	 */
	public function get_nagios_struct($prefix='') {
		$this->expand_object();
		$obj = $this->denormalize();
		return array(
//			$prefix.'id' => $obj[''],
			$prefix.'name' => $obj['host_name'],
			$prefix.'display_name' => $obj['display_name']!==""?$obj['display_name']:$obj['host_name'],
			$prefix.'alias' => $obj['alias'],
			$prefix.'address' => $obj['address'],
	//	struct hostsmember *parent_hosts;
	//	struct hostsmember *child_hosts;
	//	struct servicesmember *services;
			$prefix.'check_command' => $obj['check_command'] . ($obj['check_command_args']!==""?'!'.$obj['check_command_args']:''),
			$prefix.'check_interval' => $obj['check_interval'],
			$prefix.'retry_interval' => $obj['retry_interval'],
			$prefix.'max_attempts' => $obj['max_check_attempts'],
			$prefix.'event_handler' => $obj['event_handler'] . ($obj['event_handler_args']!==""?'!'.$obj['event_handler_args']:''),
	//	struct contactgroupsmember *contact_groups;
	//	struct contactsmember *contacts;
			$prefix.'notification_interval' => $obj['notification_interval'],
			$prefix.'first_notification_delay' => $obj['first_notification_delay'],
			$prefix.'notification_options' => $this->pack_bitmask( $obj['notification_options'], array() ), /* FIXME: to int */
//			$prefix.'hourly_value' => $obj[''],
			$prefix.'notification_period' => $obj['notification_period'],
			$prefix.'check_period' => $obj['check_period'],
			$prefix.'flap_detection_enabled' => $obj['flap_detection_enabled'],
			$prefix.'low_flap_threshold' => $obj['low_flap_threshold'],
			$prefix.'high_flap_threshold' => $obj['high_flap_threshold'],
			$prefix.'flap_detection_options' => $this->pack_bitmask( $obj['flap_detection_options'], array() ), /* FIXME: to int */
			$prefix.'stalking_options' => $this->pack_bitmask( $obj['stalking_options'], array() ), /* FIXME: to int */
			$prefix.'check_freshness' => $obj['check_freshness'],
			$prefix.'freshness_threshold' => $obj['freshness_threshold'],
			$prefix.'process_performance_data' => $obj['process_perf_data'],
			$prefix.'checks_enabled' => $obj['active_checks_enabled'],
	//		$prefix.'check_source' => $obj[''],
			$prefix.'accept_passive_checks' => $obj['passive_checks_enabled'],
			$prefix.'event_handler_enabled' => $obj['event_handler_enabled'],
			$prefix.'retain_status_information' => $obj['retain_status_information'],
			$prefix.'retain_nonstatus_information' => $obj['retain_nonstatus_information'],
			$prefix.'obsess' => $obj['obsess'],
			$prefix.'notes' => $obj['notes'],
			$prefix.'notes_url' => $obj['notes_url'],
			$prefix.'action_url' => $obj['action_url'],
			$prefix.'icon_image' => $obj['icon_image'],
			$prefix.'icon_image_alt' => $obj['icon_image_alt'],
			$prefix.'statusmap_image' => $obj['statusmap_image'],
	//		$prefix.'vrml_image' => $obj[''],
	//		$prefix.'have_2d_coords' => $obj[''], /* 2d coords needs to be processed, are avalible in nacoma */
	//		$prefix.'x_2d' => $obj[''],
	//		$prefix.'y_2d' => $obj[''],
	//		$prefix.'have_3d_coords' => $obj[''],
	//		$prefix.'x_3d' => $obj[''],
	//		$prefix.'y_3d' => $obj[''],
	//		$prefix.'z_3d' => $obj[''],
	//		$prefix.'should_be_drawn' => $obj[''],
	//	customvariablesmember *custom_variables;
	/*		$prefix.'problem_has_been_acknowledged' => $obj[''],
			$prefix.'acknowledgement_type' => $obj[''],
			$prefix.'check_type' => $obj[''],
			$prefix.'current_state' => $obj[''],
			$prefix.'last_state' => $obj[''],
			$prefix.'last_hard_state' => $obj[''],
			$prefix.'plugin_output' => $obj[''],
			$prefix.'long_plugin_output' => $obj[''],
			$prefix.'perf_data' => $obj[''],
			$prefix.'state_type' => $obj[''],
			$prefix.'current_attempt' => $obj[''],
			$prefix.'current_event_id' => $obj[''],
			$prefix.'last_event_id' => $obj[''],
			$prefix.'current_problem_id' => $obj[''],
			$prefix.'last_problem_id' => $obj[''],
			$prefix.'latency' => $obj[''],
			$prefix.'execution_time' => $obj[''],
			$prefix.'is_executing' => $obj[''],
			$prefix.'check_options' => $obj[''],
			$prefix.'notifications_enabled' => $obj[''],
			$prefix.'last_notification' => $obj[''],
			$prefix.'next_notification' => $obj[''],
			$prefix.'next_check' => $obj[''],
			$prefix.'should_be_scheduled' => $obj[''],
			$prefix.'last_check' => $obj[''],
			$prefix.'last_state_change' => $obj[''],
			$prefix.'last_hard_state_change' => $obj[''],
			$prefix.'last_time_up' => $obj[''],
			$prefix.'last_time_down' => $obj[''],
			$prefix.'last_time_unreachable' => $obj[''],
			$prefix.'has_been_checked' => $obj[''],
			$prefix.'is_being_freshened' => $obj[''],
			$prefix.'notified_on' => $obj[''],
			$prefix.'current_notification_number' => $obj[''],
			$prefix.'no_more_notifications' => $obj[''],
			$prefix.'current_notification_id' => $obj[''],
			$prefix.'check_flapping_recovery_notification' => $obj[''],
			$prefix.'scheduled_downtime_depth' => $obj[''],
			$prefix.'pending_flex_downtime' => $obj[''],
			$prefix.'last_state_history_update' => $obj[''],
			$prefix.'is_flapping' => $obj[''],
			$prefix.'flapping_comment_id' => $obj[''],
			$prefix.'percent_state_change' => $obj[''],
			$prefix.'total_services' => $obj[''],
			$prefix.'total_service_check_interval' => $obj[''],
			$prefix.'modified_attributes' => $obj[''], */

//	struct command *event_handler_ptr;
//	struct command *check_command_ptr;
//	struct timeperiod *check_period_ptr;
//	struct timeperiod *notification_period_ptr;
//	struct objectlist *hostgroups_ptr;

//	struct objectlist *exec_deps, *notify_deps;
//	struct objectlist *escalation_list;
//	struct  host *next;
//	struct timed_event *next_check_event;
				$prefix.'custom_variables' => $this->extract_custom_vars($obj)
				);
	}
}
