<?php
require_once('object_class.php');

/**
 * Use a host dependency when you want a host's status to depend
 * on other host(s) status(es).
 */
class hostdependency_class extends object_class
{
	/** object type */
	static $type = 'hostdependency';
	/** master object type */
	var $master_obj_type = 'host';
	var $master_obj_candidates = array('host' => 'dependent_host_name', 'hostgroup' => 'dependent_hostgroup_name');
	/** Variable in this object identifying the master object */
	var $master_obj_identifier = 'dependent_host_name';

	/**
	 * Make sure user-input is sane prior to invoking save/glue_objects
	 * @return bool: True on success. False on errors
	 */
	function validate_object()
	{
		global $DEBUG;
		/**
		 * service and dependent_service can legally be the same in 'new'
		 * if the user hasn't changed anything. This is the call for us to
		 * remove the 'new' object from the list.
		 */
		if($this->id==='new' && $this->obj[$this->master_obj_identifier] === $this->obj[substr($this->master_obj_identifier, 10)])
		{
			$this->validation_error('invalid dependency', 'Object cannot depend on itself');
			unset($this->obj);
		}

		if(!empty($this->obj) && $this->id==='new') {
			$this->obj[$this->master_obj_identifier] = $this->master_obj_id;
			$query 	= "SELECT host_name, hostgroup_name FROM " . $this->type .
			  " WHERE $this->master_obj_identifier = $this->master_obj_id";
			$result = pdo_exec_query($query);
			while($row = $result->fetch(PDO::FETCH_NUM)) {
				if($row[0] == $this->obj['host_name'] && $row[1] == $this->obj['hostgroup_name']) {
					# found a duplicate dependency, so remove it
					if($DEBUG) {
						echo "Duplicate " . $this->type . ".<br />\n";
						echo "<pre>\n"; print_r($this->obj); echo "</pre>\n";
					}

					unset($this->obj);
					$this->validation_error('duplicate dependency', 'A dependency between these objects already exists');
					break;
				}
			}
			unset($result);
		}
		if (!isset($this->obj) || !$this->obj){
			return false;
		}

		return parent::validate_object();
	}

	/**
	 * Computes a master object id
	 * @return False on errors, the master object id on success
	 */
	function compute_master_obj_id()
	{
		if (!$this->master_obj_type || !$this->obj)
		  return false;

		return $this->master_obj_id;
	}

	/**
	 * Prepare the object for drawing in the gui
	 */
	function prepare_draw()
	{
		parent::prepare_draw();

		$this->topic = $this->get_object_name();

		if (empty($this->master_obj_id) && !empty($this->obj[$this->master_obj_identifier])) {
			$this->master_obj_id = $this->obj[$this->master_obj_identifier];
		}

		if (empty($this->master_obj_id))
		  return;

		$clist = get_objects_for_user($this->master_obj_type);
		unset($clist[$this->master_obj_id]);
		$this->properties['special'][$this->master_obj_type.'_name'] = $clist;
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = parent::get_cmd_bar_links();
		// instead of duplicating validation from parent, just check if the validation ended up allowing copying, and only then add clone
		if (isset($cmd_links['copy']))
			$cmd_links['clone'] = '<a title="Clone host dependencies" href="edit_special.php/copy_slaves?obj_type='.$this->type.'&amp;master_obj_id='.$this->master_obj_id.'&amp;master_obj_type='.$this->master_obj_type.'&amp;obj_id='.$this->id.'">';
		return $cmd_links;
	}
}
