<?php
/**
 * Base class for PNP (graph) objects
 */
class pnp_class extends object_class
{
	const PERFDATA_DIR = "/opt/monitor/op5/pnp/perfdata/"; /**< Location of PNP perfdata */

	/**
	 * Given an object (host or service), perform seddery to return its
	 * name in the file system according to PNP
	 */
	protected static function objname_to_pnp($obj)
	{
		// more-or-less copied from PNP's sources:
		$obj = trim($obj);
		$obj = preg_replace('/[ :\/\\\]/', "_", $obj);
		return $obj;
	}

	/**
	 * Get all hosts and services that has graphs
	 *
	 * Uses livestatus, which is wrong, because livestatus reflects state, not
	 * config, but due to hostgroup services, using config would be even more
	 * wrong. Anyway, PNP won't have configured any graphs unless you've saved.
	 *
	 * This can be expensive, so will cache.
	 */
	protected function get_available_graphs() {
		static $res = NULL;
		if ($res != NULL)
			return $res;

		$ls = op5livestatus::instance();
		$hosts = $ls->query('hosts', array(
			'Filter: pnpgraph_present = 1'
			),
			array('name')
		);
		$services = $ls->query('services', array(
			'Filter: pnpgraph_present = 1'
			),
			array('host_name', 'description')
		);
		$res = array('hosts' => array(), 'services' => array());
		foreach ($hosts[1] as $row) {
			if (file_exists(self::PERFDATA_DIR.self::objname_to_pnp($row[0]).'/_HOST_.xml'))
				$res['hosts'][self::objname_to_pnp($row[0])] = $row[0];
		}
		foreach ($services[1] as $row) {
			list($host, $svc) = $row;
			$name = implode($row, ';');
			if (file_exists(self::PERFDATA_DIR.self::objname_to_pnp($host).'/'.self::objname_to_pnp($svc).'.xml'))
				$res['services'][self::objname_to_pnp($name)] = $name;
		}
		return $res;
	}

	function get_object_by_name($name)
	{
		$this->id = $name;
		return $this->get_object();
	}

	static function can_user_read($obj_type, $id)
	{
		$access = get_nacoma_access('pnp');
		return in_array(NACOMA_ACCESS_FULL_ACCESS, $access);
	}

	static function can_user_write($obj_type, $id)
	{
		$access = get_nacoma_access('pnp');
		return in_array(NACOMA_ACCESS_FULL_ACCESS, $access);
	}

	static function can_user_delete($obj_type)
	{
		$access = get_nacoma_access('pnp');
		return in_array(NACOMA_ACCESS_FULL_ACCESS, $access);
	}
}
