DROP SEQUENCE changelog_history_id_SEQ;
CREATE SEQUENCE  changelog_history_id_SEQ
  MINVALUE 1 MAXVALUE 999999999999999999999999 INCREMENT BY 1  NOCYCLE ;

DROP SEQUENCE profile_objects_id_SEQ;
CREATE SEQUENCE  profile_objects_id_SEQ
  MINVALUE 1 MAXVALUE 999999999999999999999999 INCREMENT BY 1  NOCYCLE ;

DROP SEQUENCE profiles_id_SEQ;
CREATE SEQUENCE  profiles_id_SEQ
  MINVALUE 1 MAXVALUE 999999999999999999999999 INCREMENT BY 1  NOCYCLE ;

DROP SEQUENCE role_id_SEQ;
CREATE SEQUENCE role_id_SEQ
  MINVALUE 1 MAXVALUE 999999999999999999999999 INCREMENT BY 1  NOCYCLE ;

DROP SEQUENCE role_permissions_id_SEQ;
CREATE SEQUENCE role_permissions_id_SEQ
  MINVALUE 1 MAXVALUE 999999999999999999999999 INCREMENT BY 1  NOCYCLE ;


CREATE TABLE changelog_history (
  id NUMBER(10,0) NOT NULL,
  object_type VARCHAR2(100 CHAR) NOT NULL,
  object_name VARCHAR2(255 CHAR) NOT NULL,
  oldname VARCHAR2(255 CHAR) DEFAULT NULL,
  newname VARCHAR2(255 CHAR) DEFAULT NULL,
  oldstate CLOB DEFAULT NULL,
  newstate CLOB DEFAULT NULL,
  username VARCHAR2(255 CHAR),
  time DATE DEFAULT SYSDATE
);
ALTER TABLE changelog_history ADD CONSTRAINT history_id PRIMARY KEY(id) ENABLE;
CREATE INDEX history_object_type ON changelog_history(object_type);
CREATE INDEX chlghst_username_idx ON changelog_history(username);

CREATE TABLE profile_objects (
  id NUMBER(10,0) NOT NULL,
  profile_id NUMBER(10,0) NOT NULL,
  host_name VARCHAR2(255 CHAR),
  service_description VARCHAR2(255 CHAR),
  sub_objects NUMBER(10,0) DEFAULT '0' NOT NULL,
  master_obj_id NUMBER(10,0) DEFAULT '0' NOT NULL
);
ALTER TABLE profile_objects ADD CONSTRAINT profile_objs_id PRIMARY KEY(id) ENABLE;
CREATE INDEX sub_objects ON profile_objects(sub_objects);
CREATE INDEX prototype_id ON profile_objects(profile_id);

CREATE TABLE profiles (
  id NUMBER(10,0) NOT NULL,
  name VARCHAR2(255 CHAR) NOT NULL,
  object_type VARCHAR2(25 CHAR) NOT NULL,
  object_name VARCHAR2(255 CHAR) NOT NULL,
  info CLOB
);
ALTER TABLE profiles ADD CONSTRAINT profiles_id PRIMARY KEY(id) ENABLE;
CREATE INDEX object_type ON profiles(object_type);
CREATE INDEX object_name ON profiles(object_name);

CREATE TABLE roles (
  id NUMBER(10,0) NOT NULL,
  role_name VARCHAR2(255 CHAR)
);
ALTER TABLE roles ADD CONSTRAINT role_PRIMARY PRIMARY KEY(id) ENABLE;

CREATE TABLE role_permissions (
  id NUMBER(10,0) NOT NULL,
  role_id NUMBER(10,0) NOT NULL,
  object_type VARCHAR2(100 CHAR) NOT NULL,
  object_field VARCHAR2(100 CHAR) DEFAULT NULL,
  rule CLOB NOT NULL
);
ALTER TABLE role_permissions ADD CONSTRAINT role_permissions_PRIMARY PRIMARY KEY(id) ENABLE;
CREATE INDEX role_permissions_id ON role_permissions(role_id);

CREATE TABLE user_roles (
  username VARCHAR2(255 CHAR) NOT NULL,
  role NUMBER(10,0) NOT NULL
);
CREATE INDEX user_roles_role ON user_roles(role);
CREATE INDEX user_roles_user ON user_roles(username);


CREATE OR REPLACE TRIGGER changelog_history_id_TRG BEFORE INSERT OR UPDATE ON changelog_history
FOR EACH ROW
DECLARE
v_newVal NUMBER(12) := 0;
v_incval NUMBER(12) := 0;
BEGIN
  IF INSERTING AND :new.id IS NULL THEN
    SELECT  changelog_history_id_SEQ.NEXTVAL INTO v_newVal FROM DUAL;
    -- If this is the first time this table have been inserted into (sequence == 1)
    IF v_newVal = 1 THEN
      -- get the max indentity value from the table
      SELECT NVL(max(id),0) INTO v_newVal FROM changelog_history;
      v_newVal := v_newVal + 1;
      -- set the sequence to that value
      LOOP
           EXIT WHEN v_incval>=v_newVal;
           SELECT changelog_history_id_SEQ.nextval INTO v_incval FROM dual;
      END LOOP;
    END IF;
    -- used to emulate LAST_INSERT_ID()
    -- mysql_utilities.identity := v_newVal;
   -- assign the value from the sequence to emulate the identity column
   :new.id := v_newVal;
  END IF;
END;

/

CREATE OR REPLACE TRIGGER profile_objects_id_TRG BEFORE INSERT OR UPDATE ON profile_objects
FOR EACH ROW
DECLARE
v_newVal NUMBER(12) := 0;
v_incval NUMBER(12) := 0;
BEGIN
  IF INSERTING AND :new.id IS NULL THEN
    SELECT  profile_objects_id_SEQ.NEXTVAL INTO v_newVal FROM DUAL;
    -- If this is the first time this table have been inserted into (sequence == 1)
    IF v_newVal = 1 THEN
      -- get the max indentity value from the table
      SELECT NVL(max(id),0) INTO v_newVal FROM profile_objects;
      v_newVal := v_newVal + 1;
      -- set the sequence to that value
      LOOP
           EXIT WHEN v_incval>=v_newVal;
           SELECT profile_objects_id_SEQ.nextval INTO v_incval FROM dual;
      END LOOP;
    END IF;
    -- used to emulate LAST_INSERT_ID()
    -- mysql_utilities.identity := v_newVal;
   -- assign the value from the sequence to emulate the identity column
   :new.id := v_newVal;
  END IF;
END;

/

CREATE OR REPLACE TRIGGER profiles_id_TRG BEFORE INSERT OR UPDATE ON profiles
FOR EACH ROW
DECLARE
v_newVal NUMBER(12) := 0;
v_incval NUMBER(12) := 0;
BEGIN
  IF INSERTING AND :new.id IS NULL THEN
    SELECT  profiles_id_SEQ.NEXTVAL INTO v_newVal FROM DUAL;
    -- If this is the first time this table have been inserted into (sequence == 1)
    IF v_newVal = 1 THEN
      -- get the max indentity value from the table
      SELECT NVL(max(id),0) INTO v_newVal FROM profiles;
      v_newVal := v_newVal + 1;
      -- set the sequence to that value
      LOOP
           EXIT WHEN v_incval>=v_newVal;
           SELECT profiles_id_SEQ.nextval INTO v_incval FROM dual;
      END LOOP;
    END IF;
    -- used to emulate LAST_INSERT_ID()
    -- mysql_utilities.identity := v_newVal;
   -- assign the value from the sequence to emulate the identity column
   :new.id := v_newVal;
  END IF;
END;

/

CREATE OR REPLACE TRIGGER role_id_TRG BEFORE INSERT OR UPDATE ON role
FOR EACH ROW
DECLARE
v_newVal NUMBER(12) := 0;
v_incval NUMBER(12) := 0;
BEGIN
  IF INSERTING AND :new.id IS NULL THEN
    SELECT  role_id_SEQ.NEXTVAL INTO v_newVal FROM DUAL;
    -- If this is the first time this table have been inserted into (sequence == 1)
    IF v_newVal = 1 THEN
      -- get the max indentity value from the table
      SELECT NVL(max(id),0) INTO v_newVal FROM role;
      v_newVal := v_newVal + 1;
      -- set the sequence to that value
      LOOP
           EXIT WHEN v_incval>=v_newVal;
           SELECT role_id_SEQ.nextval INTO v_incval FROM dual;
      END LOOP;
    END IF;
    -- used to emulate LAST_INSERT_ID()
    -- mysql_utilities.identity := v_newVal;
   -- assign the value from the sequence to emulate the identity column
   :new.id := v_newVal;
  END IF;
END;

/

CREATE OR REPLACE TRIGGER role_permissions_id_TRG BEFORE INSERT OR UPDATE ON role_permissions
FOR EACH ROW
DECLARE
v_newVal NUMBER(12) := 0;
v_incval NUMBER(12) := 0;
BEGIN
  IF INSERTING AND :new.id IS NULL THEN
    SELECT  role_permissions_id_SEQ.NEXTVAL INTO v_newVal FROM DUAL;
    -- If this is the first time this table have been inserted into (sequence == 1)
    IF v_newVal = 1 THEN
      -- get the max indentity value from the table
      SELECT NVL(max(id),0) INTO v_newVal FROM role_permissions;
      v_newVal := v_newVal + 1;
      -- set the sequence to that value
      LOOP
           EXIT WHEN v_incval>=v_newVal;
           SELECT role_permissions_id_SEQ.nextval INTO v_incval FROM dual;
      END LOOP;
    END IF;
    -- used to emulate LAST_INSERT_ID()
    -- mysql_utilities.identity := v_newVal;
   -- assign the value from the sequence to emulate the identity column
   :new.id := v_newVal;
  END IF;
END;

/

INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'host', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'service', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'contact', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'hostgroup', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'servicegroup', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'contactgroup', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'template', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'host_template', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'service_template', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'contact_template', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'command', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'timeperiod', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'FILE', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'servicedependency', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'serviceescalation', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'hostdependency', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'hostescalation', 'allow');
INSERT INTO role_permissions (role_id, object_type, rule) VALUES (1, 'pnp', 'allow');
COMMIT;
