<?php
// because there's some stuff that just too obscure to create separate test files for

require_once('devtools/tapunit.php');
require_once('include/probe.inc.php');

class MiscTest extends TapUnit
{
	function testGetMerlinNodes() {
		$test_node_show_data = <<<EOS

# poller_name1
ADDRESS=poller_address1
TYPE=poller
PORT=15551
HOSTGROUP=hostgroup_1
NAME=poller_name1

# poller_name2
ADDRESS=poller_address2
TYPE=poller
PORT=15551
HOSTGROUP=hostgroup_4
NAME=poller_name2

# peer_name
ADDRESS=peer_address
TYPE=peer
PORT=15551
NAME=peer_name
EOS
		;
		set_user('host_7', array('op5_admins'));
		$output = parse_merlin_nodes($test_node_show_data);
		$this->ok_eq($output, array('This server', 'poller_name1', 'poller_name2', 'peer_name'), 'get_merlin_nodes should return all nodes to super-users');

		set_user('contact_limited', array('contact_limited'));
		$output = parse_merlin_nodes($test_node_show_data);
		$this->ok_eq($output, array('This server', 'poller_name2', 'peer_name'), "get_merlin_nodes shouldn't return all nodes to limited users, but it should return some");
	}

	function testManagementPackManagementValidation()
	{
		$invalid_packs = array(
			null,
			array('management_pack' => array('management_pack' => array('management_pack_name' => 'foo'))),
			array('management_pack' => array('commands' => array())),
			'this is not a management pack',
		);

		foreach ($invalid_packs as $pack) {
			try {
				management_pack_management_class::validate_input($pack);
				$this->fail('Validation should fail on invalid pack '.var_export($pack, true));
			} catch (ManagementPackManagementValidationException $ex) {
				$this->pass('Validation failed on invalid pack '.var_export($pack, true));
			}
		}
	}


	function trixy_check_helper($pack, $descr)
	{
		$mpm = new management_pack_management_class();
		$res = $mpm->is_imported($pack);
		$this->ok(is_array($res), "Expected management pack $descr to be trixsy, got $res");
		$res = $mpm->import($pack);
		$this->ok_id($res, false, "Importing the management pack $descr force-free should complain, got ".
			var_export(is_object($res)?$res->validation_error_list:$res, true));
		$res = $mpm->import($pack, true);
		$this->ok($res > 0, "Importing the management pack $descr with force should work, got ".
			var_export(is_object($res)?$res->validation_error_list:$res, true));
		$this->ok_id($mpm->is_imported($pack), 1, "The management pack $descr should be considered imported when asking again");
	}
	function unimported_check_helper($pack, $descr)
	{
		$mpm = new management_pack_management_class();
		$this->ok_id($mpm->is_imported($pack), 0, "The management pack $descr should not be imported");
		$res = $mpm->import($pack);
		$this->ok($res > 0, "Importing the management pack $descr should work, got ".
			var_export(is_object($res)?$res->validation_error_list:$res, true));
		$this->ok_id($mpm->is_imported($pack), 1, "The management pack $descr should be considered imported when asking again");
	}
	function cleanup_helper($pack)
	{
		set_user('host_7', array('op5_admins'));
		foreach ($pack['commands'] as $cmd) {
			$command = new command_class($cmd['command_name']);
			$this->ok($command->banish_object(), 'Deleting command should work, got '.
				$command->get_deletion_message());
		}
		foreach ($pack['services'] as $svc) {
			$service = new service_class($svc['hostgroup_name'].';'.$svc['service_description']);
			$this->ok($service->banish_object(), 'Deleting service should work, got '.
				$service->get_deletion_message());
		}
		$mgmt = new management_pack_class($pack['management_pack']['management_pack_name']);
		$this->ok($mgmt->banish_object(), 'Deleting management pack should work, got '.
			$mgmt->get_deletion_message());
		$hg = new hostgroup_class($pack['hostgroup']['hostgroup_name']);
		$this->ok($hg->banish_object(), 'Deleting hostgroup should work, got '.
			$hg->get_deletion_message());
	}

	function testManagementPackIsImported()
	{
		set_user('host_7', array('op5_admins'));
		$real_pack = array(
			'management_pack' => array(
				'management_pack_name' => 'TEST_PACK',
				'description' => "This is not the pack you're looking for",
				'icon' => 'server.png',
				'hostgroup' => 'TEST_HG',
				'_VAR' => 'value',
			),
			'hostgroup' => array(
				'hostgroup_name' => 'TEST_HG',
				'notes' => 'what?',
			),
			'services' => array(
				array(
					'service_description' => 'TEST_SVC1',
					'hostgroup_name' => 'TEST_HG',
					'check_command' => 'TEST_COMMAND1',
					'notes' => 'no reason',
				),
				array(
					'service_description' => 'TEST_SVC2',
					'hostgroup_name' => 'TEST_HG',
					'check_command' => 'TEST_COMMAND2',
				),
			),
			'commands' => array(
				array(
					'command_name' => 'TEST_COMMAND1',
					'command_line' => '/bin/echo Weeee',
				),
				array(
					'command_name' => 'TEST_COMMAND2',
					'command_line' => '/bin/echo Wooo',
				),
			),
		);

		$mpm = new management_pack_management_class();
		$this->ok_id($mpm->is_imported($real_pack), 0, 'The management pack should not be imported');
		$res = $mpm->import($real_pack);
		$this->ok($res > 0, 'Importing the management pack should work, got '.
			var_export(is_object($res)?$res->validation_error_list:$res, true));
		$this->ok_id($mpm->is_imported($real_pack), 1, 'The management pack should be considered imported when asking again');
		$res = $mpm->import($real_pack);
		$this->ok($res > 0, 'Re-importing the management pack should work, got '.
			var_export(is_object($res)?$res->validation_error_list:$res, true));

		$real_pack['services'][]= array(
			'service_description' => 'third svc',
			'hostgroup_name' => 'TEST_HG',
			'check_command' => 'TEST_COMMAND1',
		);
		$this->unimported_check_helper($real_pack, 'with new service');
		$real_pack['commands'][]= array(
			'command_name' => 'TEST_COMMAND3',
			'command_line' => '/bin/echo Waaa',
		);
		$this->unimported_check_helper($real_pack, 'with new command');
		$real_pack['management_pack']['_VAR2'] = 'other value';
		$this->unimported_check_helper($real_pack, 'with new custom var on pack');
		$real_pack['management_pack']['description'] = "Why can't we be friends?";
		$this->unimported_check_helper($real_pack, 'with changed description on pack');

		$real_pack['commands'][0]['command_line'] = 'I changed you';
		$this->trixy_check_helper($real_pack, 'with changed command');
		$real_pack['services'][0]['notes'] = 'Pikachu, I choose you';
		$this->trixy_check_helper($real_pack, 'with changed service');
		$real_pack['hostgroup']['notes'] = 'This note was intentionally left blank.';
		$this->trixy_check_helper($real_pack, 'with changed hostgroup');

		$real_pack['management_pack']['_VAR2'] = 'value';
		unset($real_pack['management_pack']['_VAR']);
		$this->unimported_check_helper($real_pack, 'with changed description on pack');
		$mp = new management_pack_class($real_pack['management_pack']['management_pack_name']);
		$this->ok_id($mp->custom_vars['_VAR2'], 'value', "After renaming custom var, the new name is set");
		$this->ok(!isset($mp->custom_vars['_VAR']), "After renaming custom var, the old name is not set");

		$this->cleanup_helper($real_pack);

		# It's important to bail here for new packs, or 6.2 upgrades would break stuff
		$pack_existing_hg = $real_pack;
		$pack_existing_hg['management_pack']['management_pack_name'] = 'another test pack';
		$pack_existing_hg['hostgroup']['hostgroup_name'] = 'hostgroup_1';
		$res = $mpm->is_imported($pack_existing_hg);
		$this->ok(is_array($res), "The new management pack with preexisting hostgroup is trixsy, so expected exception, got ".$res);
		$this->ok_id($mpm->import($pack_existing_hg), false, "The new management pack with preexisting hostgroup can't be imported without force");
	}

	function testMgmtPackImportWithLimitedRights()
	{
		$mpm = new management_pack_management_class();
		$pack = array(
			'management_pack' => array(
				'management_pack_name' => 'CLI_TEST_PACK',
				'description' => "This is not the pack you're looking for",
				'icon' => 'server.png',
				'hostgroup' => 'CLI_TEST_HG',
				'_VAR' => 'value',
			),
			'hostgroup' => array(
				'hostgroup_name' => 'CLI_TEST_HG',
				'notes' => 'what?',
			),
			'services' => array(
				array(
					'service_description' => 'CLI_TEST_SVC1',
					'hostgroup_name' => 'CLI_TEST_HG',
					'check_command' => 'CLI_TEST_COMMAND1',
					'notes' => 'no reason',
				),
			),
			'commands' => array(
				array(
					'command_name' => 'CLI_TEST_COMMAND1',
					'command_line' => '/bin/echo Weeee',
				),
			),
		);
		$ug = new usergroup_class('mgmt_limited_group');
		$ug->obj['access_rights'] = array('management_pack_edit_all', 'management_pack_view_all', 'management_pack_add_delete');
		$this->ok($ug->save_object(), 'New usergroup should be saved, but got: '.var_export($ug->validation_error_list, true));
		set_user('contact_limited', array('mgmt_limited_group'));

		$res = $mpm->import($pack, true);
		$this->ok(is_object($res), "When trying to import a management pack with only management pack rights, I should get an object error back");
		$this->ok(!empty($res->validation_error_list), "When trying to import a management pack with only management pack rights, the return should have validation errors");

		$obj = new management_pack_class('CLI_TEST_PACK');
		$this->ok_id($obj->id, false, "After lacking permissions to import everything, the pack I did have access to import shouldn't be imported");

		$ug->obj['access_rights'] = array('management_pack_edit_all', 'management_pack_view_all', 'management_pack_add_delete', 'command_add_delete');
		$this->ok($ug->save_object(), 'New usergroup should be saved, but got: '.var_export($ug->validation_error_list, true));
		set_user('contact_limited', array('mgmt_limited_group'));

		$res = $mpm->import($pack, true);
		$this->ok(is_object($res), "When trying to import a management pack with only management pack + command rights, I should get an object error back");
		$this->ok(!empty($res->validation_error_list), "When trying to import a management pack with only management pack + command rights, the return should have validation errors");

		$obj = new management_pack_class('CLI_TEST_PACK');
		$this->ok_id($obj->id, false, "After lacking permissions to import everything, the pack I did have access to import shouldn't be imported");
		$obj = new command_class('CLI_TEST_COMMAND1');
		$this->ok_id($obj->id, false, "After lacking permissions to import everything, the command I did have access to import shouldn't be imported");

		$this->cleanup_helper($pack);
	}

	function testManagementPackCli()
	{
		$pack = array(
			'management_pack' => array(
				'management_pack_name' => 'CLI_TEST_PACK',
				'description' => "This is not the pack you're looking for",
				'icon' => 'server.png',
				'hostgroup' => 'CLI_TEST_HG',
				'_VAR' => 'value',
			),
			'hostgroup' => array(
				'hostgroup_name' => 'CLI_TEST_HG',
				'notes' => 'what?',
			),
			'services' => array(
				array(
					'service_description' => 'CLI_TEST_SVC1',
					'hostgroup_name' => 'CLI_TEST_HG',
					'check_command' => 'CLI_TEST_COMMAND1',
					'notes' => 'no reason',
				),
				array(
					'service_description' => 'CLI_TEST_SVC2',
					'hostgroup_name' => 'CLI_TEST_HG',
					'check_command' => 'CLI_TEST_COMMAND2',
				),
			),
			'commands' => array(
				array(
					'command_name' => 'CLI_TEST_COMMAND1',
					'command_line' => '/bin/echo Weeee',
				),
				array(
					'command_name' => 'CLI_TEST_COMMAND2',
					'command_line' => '/bin/echo Wooo',
				),
			),
		);
		$name = tempnam("/usr/share/monitor/management-packs", "testpack").'.json';
		$this->ok(file_put_contents($name, json_encode(array('management_pack' => $pack, 'vendor'=>'', 'changes'=>array(array('version'=>1,'description'=>'no changes'))))) !== 'false', "Writing a test pack to file should work");
		exec("php ./mgmtpackimport.php --import --name ".basename($name, '.json'), $out, $return);
		$this->ok_id($return, 0, "Exit code from importing mgmtpack should be 0");
		$this->ok_empty(trim(implode("\n", $out)), "Expected import to be OK");
		$this->cleanup_helper($pack);

		$out=false;
		$pack['hostgroup']['hostgroup_name'] = 'hostgroup_1';
		$hg = new hostgroup_class($pack['hostgroup']['hostgroup_name']);
		$this->ok($hg->id !== false, "The hostgroup used for testing for preexisting hostgroup should exist");
		$this->ok(file_put_contents($name, json_encode(array('management_pack' => $pack, 'vendor'=>'', 'changes'=>array(array('version'=>1,'description'=>'no changes'))))) !== 'false', "Writing a test pack to file should work");
		exec("php ./mgmtpackimport.php --import --name ".basename($name, '.json'), $out, $return);
		$this->ok_id($return, 1, "Exit code from importing mgmtpack with already-existing hostgroup should be 1");
		$this->ok_eq($out[0], "Cannot import management pack 'CLI_TEST_PACK', because it has been modified. You can try to force the import, but that might cause checks to change or fail.", "If the hostgroup already exist, we should not import without warning");
		unlink($name);
	}

	function tearDown()
	{
		pdo_exec_query('DELETE FROM changelog');
	}
}

if (empty($main_tap))
	$main_tap = new phptap('Object tests');

runUnit('MiscTest', $main_tap);
