<?php

require_once('devtools/tapunit.php');
require_once('classes/object_class.php');

class SpecialConfigs extends TapUnit
{
	function setUp()
	{
		global $slave_obj_array, $Object_Index_Table, $Objects_For_User;
		global $Object_Reference_List, $Reverse_Object_Reference_table;
		global $import_warnings, $import_errors, $index, $num_objs, $obj_array;
		global $master_list;
		global $custom_vars, $Template_Cache;
		$Object_Index_Table = false;
		$Objects_For_User = false;
		$Object_Reference_List = false;
		$Reverse_Object_Reference_Table = array();
		$master_list = false;
		$slave_obj_array = array();
		$import_warnings = array();
		$import_errors = array();
		$index = array();
		$num_objs = array();
		$obj_array = array();
		$custom_vars = array();
		$Template_Cache = array();
	}

	function config_equal($pre_fname, $post_fname)
	{
		$empty_file = array("###############################################################################\n",
		                    "#  Generated by op5 Monitor webconfiguration exporter\n",
		                    "#\n",
		                    "# Exported <date> by <user>\n",
		                    "#\n\n");
		if ($pre_fname === null)
			$pre = $empty_file;
		else
			$pre = file($pre_fname);
		if ($post_fname === null)
			$post = $empty_file;
		else
			$post = file($post_fname);
		# line 4 is the one with the export date. Unset:
		if (isset($pre[3]))
			unset($pre[3]);
		if (isset($post[3]))
			unset($post[3]);
		$pre = implode("", $pre);
		$post = implode("", $post);
		if ($pre == $post) {
			$this->pass("Config file is the same after import and export");
		} else {
			$this->fail("Config file differ from expected after import and export:\n$pre_fname:\n$pre\n$post_fname\n$post\n");
		}
	}

	/**
	 * When there was only one serviceextinfo, and that one serviceextinfo didn't
	 * have a parent, things used to break.
	 */
	function testImportBuggyServiceextinfo()
	{
		global $HOME;
		# disable asserts during this test - the hostextinfo triggers (quite correct) errors
		assert_options(ASSERT_ACTIVE, 0);
		$cfg = $HOME.'minimal-buggy-extinfo.cfg';
		$this->ok(import_config('phptap', $cfg), "import_config('phptap', '$cfg')");
		assert_options(ASSERT_ACTIVE, 1);
	}

	function testExportAfterDelete()
	{
		global $HOME;
		$empty_pre = $HOME.'deleted-dependencies.cfg';
		$hg_dep_pre = $HOME.'related-dependencies-hg.cfg';
		$host_dep_pre = $HOME.'related-dependencies-host.cfg';
		$pre_cfg = array($empty_pre, $hg_dep_pre, $host_dep_pre);
		$post_cfg = array();
		foreach ($pre_cfg as $cfg) {
			$post_cfg[] = $cfg.'.new';
		}
		$this->ok(import_config('phptap', $pre_cfg), "import_config('phptap', '$pre_cfg')");
		$host = new host_class('the_host');
		$this->ok($host->id && is_numeric($host->id), 'Host was found');
		$host->banish_object();
		$cg = new contactgroup_class('the_contactgroup');
		$this->ok($cg->id && is_numeric($cg->id), 'Contactgroup was found');
		$cg->banish_object();

		foreach ($post_cfg as $cfg)
			touch($cfg);

		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->config_equal($empty_pre.'.new', null);
		$this->config_equal($hg_dep_pre.'.new', $hg_dep_pre.'.expected');
		$this->config_equal($host_dep_pre.'.new', $host_dep_pre.'.expected');
	}

	function testExportAfterOtherCreate()
	{
		global $HOME;
		$empty_pre = $HOME.'deleted-dependencies.cfg';
		$hg_dep_pre = $HOME.'related-dependencies-hg.cfg';
		$host_dep_pre = $HOME.'related-dependencies-host.cfg';
		$pre_cfg = array($hg_dep_pre, $host_dep_pre, $empty_pre);
		$post_cfg = array();
		foreach ($pre_cfg as $cfg) {
			$post_cfg[] = $cfg.'.new';
		}
		$this->ok(import_config('phptap', $pre_cfg, true), "import_config('phptap', '$pre_cfg')");
		set_user('the_contact', array('op5_admins'));
		$this->setUp();
		$h = new host_class();
		$h->normalize(array(
			'host_name' => 'unexported_host',
			'file_id' => $host_dep_pre,
			'template' => 0,
			'max_check_attempts' => 1,
			'notification_interval' => 1,
			'check_period' => 0,
			'notification_period' => 0,
		));
		$this->ok($h->save_object(), 'host should be saved');
		$this->ok_empty($h->validation_error_list, 'There should be no errors saving');
		$svc = new service_class();
		$svc->normalize(array(
			'host_name' => 'unexported_host',
			'service_description' => 'not_exported',
			'file_id' => $hg_dep_pre,
			'template' => 0,
			'max_check_attempts' => 1,
			'notification_interval' => 1,
			'check_period' => 0,
			'check_command' => 0,
			'check_interval' => 1,
			'retry_interval' => 1,
			'notification_period' => 0,
		));
		$this->ok($svc->save_object(), 'service should be saved');
		$this->ok_empty($svc->validation_error_list, 'There should be no errors saving');
		set_user('host_7', array('op5_admins'));
		$this->setUp();
		$hg = new hostgroup_class('the_hostgroup');
		$hg->obj['notes'] = 'foo';
		$this->ok($hg->save_object(), 'hostgroup should be savable');
		$this->ok_empty($hg->validation_error_list, 'hostgroup should not have validation errors');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$chng = new Changelog();
		$ret = do_export($chng);
		$this->ok_id($ret, true, 'Config export was successful');
		$this->config_equal($hg_dep_pre.'.new', $hg_dep_pre.'.expected2');
	}

	/**
	 * Test for bug #4999
	 */
	function testTrailingComma()
	{
		global $HOME;
		$pre = $HOME.'trailing-comma.cfg';
		$post = $HOME.'trailing-comma.cfg.expected';
		$this->ok(import_config('phptap', $pre), "import_config('phptap', '$pre')");

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$ret = do_export(true);
		$this->ok_id($ret, true, 'Config export was successful');
		$this->config_equal($pre.'.new', $post);
	}

	/**
	 * Test for bug #5105
	 */
	function testMinimizeConfig()
	{
		global $HOME;
		$pre = $HOME.'minimize-template.cfg';
		$post = $HOME.'minimize-template.cfg.expected';
		$this->ok(import_config('phptap', $pre), "import_config('phptap', '$pre')");

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$host = new host_class('test');
		$this->ok($host->save_object(), 'Save object is ok');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->config_equal($pre.'.new', $post);
	}

	/**
	 * The bug is that get_object can't fetch any more relations than it fetches objects
	 */
	function testChangeUnrelated()
	{
		global $HOME;
		$pre_base = $HOME.'hostwithservices.cfg';
		$pre_deps = $HOME.'separate_service.cfg';
		$this->ok(import_config('phptap', array($pre_base, $pre_deps)), "import_config('phptap', array($pre_base, $pre_deps))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre_base.'.new');
		touch($pre_deps.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$svc = new service_class('base_host;separate_service');
		$svc->obj['notes'] = 'foo';
		$this->ok($svc->save_object(), 'Save object is ok');
		$this->ok_empty($svc->validation_error_list, 'No save failures');
		unset($svc->obj['notes']);
		$this->ok($svc->save_object(), 'Save object is ok');
		$this->ok_empty($svc->validation_error_list, 'No save failures');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$this->config_equal($pre_deps.'.new', $pre_deps);
	}

	function testChangeChild()
	{
		global $HOME;
		$pre_first = $HOME.'emptyhostinafile1.cfg';
		$pre_second = $HOME.'emptyhostinafile2.cfg';
		$this->ok(import_config('phptap', array($pre_first, $pre_second)), "import_config('phptap', array($pre_first, $pre_second))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre_first.'.new');
		touch($pre_second.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$host1 = new host_class('hostinfile1');
		$this->ok(!empty($host1->obj), 'host has data');

		$host2 = new host_class('hostinfile2');
		$this->ok(!empty($host2->obj), 'host has data');

		$host2->obj['parents'] = array($host1->id);
		$this->ok($host2->save_object(), 'host with parents saved');
		$this->ok_empty($host2->validation_error_list, 'Error list was empty');
		$host2 = new host_class('hostinfile2');
		$this->ok_eq($host2->obj['parents'], array($host1->id), 'Parent is still there after saving');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$host1 = new host_class('hostinfile1');
		$this->ok_eq($host1->obj['children'], array($host2->id), 'Parent has child after exporting');

		$host1->obj['host_name'] = 'hostinfilewithnewname';
		$this->ok($host1->save_object(), 'host with parents saved');
		$this->ok_empty($host1->validation_error_list, 'Error list was empty');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$this->config_equal($pre_first.'.new', $pre_first.'.expected');
		$this->config_equal($pre_second.'.new', $pre_second.'.expected');
	}

	/**
	 * Test for bug #7452
	 */
	function testAddContactGroupMember()
	{
		global $HOME;
		$pre_first = $HOME.'testAddContactGroupMember-cg.cfg';
		$pre_second = $HOME.'testAddContactGroupMember-c.cfg';

		$this->ok(import_config('phptap', array($pre_first, $pre_second)), "import_config('phptap', array($pre_first, $pre_second))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre_first.'.new');
		touch($pre_second.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$cg = new contactgroup_class('the_contactgroup');
		$this->ok(!empty($cg->obj), 'Contactgroup should be loaded with data');
		$c = new contact_class('the_contact');
		$this->ok(!empty($c->obj), 'Contact should be loaded with data');
		$cg->obj['members'] = array($c->id);

		$this->ok($cg->save_object(), 'Contactgroup with new member should be saved correctly');
		$this->ok_empty($cg->validation_error_list, 'Contactgroup should have empty error list after saving');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$this->config_equal($pre_first.'.new', $pre_first.'.expected');
		$this->config_equal($pre_second.'.new', $pre_second.'.expected');

		$hg = new hostgroup_class('the_hostgroup');
		$this->ok(!empty($hg->obj), 'Hostgroup should be loaded with data');
		$h = new host_class('the_host');
		$this->ok(!empty($h->obj), 'Host should be loaded with data');
		$hg->obj['members'] = array($h->id);

		$this->ok($hg->save_object(), 'Hostgroup with new member should be saved correctly');
		$this->ok_empty($hg->validation_error_list, 'Hostgroup should have empty error list after saving');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$this->config_equal($pre_first.'.new', $pre_first.'.expected2');
		$this->config_equal($pre_second.'.new', $pre_second.'.expected2');
	}

	function testTemplateInTemplate()
	{
		global $HOME;
		$pre = $HOME.'template-in-template.cfg';

		$this->ok(import_config('phptap', array($pre)), "import_config('phptap', array($pre))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		# customer use case
		$parent = new service_template_class('default-service');
		$svctmpl = new service_template_class();
		$svctmpl->obj['name'] = 'ping';
		$svctmpl->obj['active_checks_enabled'] = 1;
		$svctmpl->obj['check_freshness'] = 1;
		$svctmpl->obj['freshness_threshold'] = 1300;
		$svctmpl->obj['notes'] = 'Ping tjänst - ICMP echo';
		$svctmpl->obj['passive_checks_enabled'] = 1;
		$svctmpl->obj['template'] = $parent->id;

		$this->ok($svctmpl->save_object(), 'New service template with parent template should be saved correctly');
		$this->ok_empty($svctmpl->validation_error_list, 'New service template should have empty error list after saving');

		# test other templates while we're at it
		$parent = new host_template_class('default-host-template');
		$hosttmpl = new host_template_class();
		$hosttmpl->obj['name'] = 'monitor';
		$hosttmpl->obj['template'] = $parent->id;

		$this->ok($hosttmpl->save_object(), 'New host template with parent template should be saved correctly');
		$this->ok_empty($hosttmpl->validation_error_list, 'New host template should have empty error list after saving');

		$parent = new contact_template_class('default-contact');
		$contacttmpl = new contact_template_class();
		$contacttmpl->obj['name'] = 'monitor';
		$contacttmpl->obj['template'] = $parent->id;

		$this->ok($contacttmpl->save_object(), 'New contact template with parent template should be saved correctly');
		$this->ok_empty($contacttmpl->validation_error_list, 'New contact template should have empty error list after saving');

		# test non-templates using these templates
		$host = new host_class();
		$host->obj['host_name'] = 'test_host';
		$host->obj['template'] = $hosttmpl->id;
		$host->obj['notification_period'] = 1;
		$this->ok($host->save_object(), 'New host with template should be saved correctly');
		$this->ok_empty($host->validation_error_list, 'New host should have empty error list after saving');

		$svc = new service_class();
		$svc->obj['host_name'] = $host->id;
		$svc->obj['service_description'] = 'test_description';
		$svc->obj['template'] = $svctmpl->id;
		$svc->obj['notification_period'] = 1;
		$svc->obj['check_period'] = 1;
		$svc->obj['check_command'] = 1;
		$this->ok($svc->save_object(), 'New service with template should be saved correctly');
		$this->ok_empty($svc->validation_error_list, 'New service should have empty error list after saving');

		$contact = new contact_class();
		$contact->obj['contact_name'] = 'test_contact';
		$contact->obj['alias'] = 'test_contact';
		$contact->obj['template'] = $contacttmpl->id;
		$this->ok($contact->save_object(), 'New contact with template should be saved correctly');
		$this->ok_empty($contact->validation_error_list, 'New contact should have empty error list after saving');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$this->config_equal($pre.'.new', $pre.'.expected');
	}

	function testPervertedAccess()
	{
		global $HOME;
		$pre = $HOME.'objecttree.cfg';

		$this->ok(import_config('phptap', array($pre)), "import_config('phptap', array($pre))");
		global $IMPORTING;
		$IMPORTING=false;

		$ug = new usergroup_class('all_by_contact');
		$ug->obj['access_rights'] = array(
			'configuration_information',
			'host_view_contact',
			'host_template_view_all',
			'hostgroup_view_contact',
			'service_view_contact',
			'service_template_view_all',
			'servicegroup_view_contact',
			'contact_view_contact',
			'contact_template_view_all',
			'contactgroup_view_contact',
		);
		$this->ok($ug->save_object(), 'New usergroup should be saved, but got: '.var_export($ug->validation_error_list, true));

		set_user('the_contact', array('all_by_contact'));
		/* I changed user, let's flush caches */
		$this->setUp();
		$objects = array(
			'host' => array(
				'branch1_host',
				'branch2_host',
				'branch4_host',
			),
			'host_template' => array(
				'branch1_htmpl',
				'branch2_htmpl',
				'branch4_htmpl',
			),
			'hostgroup' => array(
				'branch2_hg',
				'branch4_hgroup2',
				'branch4_hgroup1',
			),
			'service' => array(
				'branch3_host;branch3_svc',
				'branch1_host;branch1_svc',
				'branch2_host;branch2_svc',
				'branch4_svc_host;branch4_svc',
			),
			'service_template' => array(
				'branch3_stmpl1',
				'branch3_stmpl2',
				'branch4_stmpl',
			),
			'servicegroup' => array(
				'branch1_sg1',
				'branch1_sg2',
				'branch3_sgroup',
			),
			'contact' => array(
				'the_contact',
				'branch2_contact',
			),
			'contact_template' => array(
				'branch4_ctmpl1',
				'branch4_ctmpl2',
			),
			'contactgroup' => array(
				'parent_cgroup',
				'grandparent_cgroup',
				'branch2_cgroup',
			),
		);
		foreach ($objects as $type => $objs) {
			$class = $type.'_class';
			foreach ($objs as $objname) {
				$obj = new $class($objname);
				$this->ok($obj->can_user_read($type, $obj->id), "$type $objname should load properly");
			}
			$obj = new $class('uncon_'.$type);
			$this->ok(!$obj->id || !$obj->can_user_read($type, $obj->id), "Unconnected $type should not load properly");
		}
	}
}

if (empty($main_tap))
	$main_tap = new phptap('Object tests');

runUnit('SpecialConfigs', $main_tap);
