<?php

require_once('select.php');

/**
 * Multi-select input widget
 */
class mselect_widget extends select_widget {
	/**
	 * Creates 2 multiple select boxes with unselected alternatives
	 * on the left and selected on the right.
	 * @param $tokenvalues array: Alternatives to list, in key => value form
	 * @param $dflt The default-selected items
	 * @param $select_name html form name
	 */
	public static function make_multi_selection($tokenvalues, $dflt, $select_name = 'selection')
	{
		global $DEBUG;

		/* if array is empty or only has one object, we tell the user so */
		if(!is_array($tokenvalues) || !count($tokenvalues)) {
			echo "None configured\n";
			return true;
		}

		$noprint = false;
		if(func_num_args() > 3) {
			$noprint = func_get_arg(3);
		}

		$size = count($tokenvalues);
		if($size > 20) $size = 20;


		// create a container (table) to hold the 2 select boxes with
		// the 2 buttons in between
		echo "<table class=\"nopad\">\n";
		if (sizeof($tokenvalues)>$size){
			echo '<tr><td colspan="3" style="border: 0px">';
			self::addFilterlistButtons(sizeof($tokenvalues));
			echo '</td></tr>';
		}
		echo "\n<tr><td style=\"border: 0px\">";

		// set up the selection and iterate the values
		// Left box = available (unselected) options
		if (($pos = strpos($select_name, '[')) === false)
			$select_tmp = $select_name.'_tmp';
		else
			$select_tmp = substr($select_name, 0, $pos) . '_tmp' . substr($select_name, $pos);
		$id = preg_replace('/[^a-z0-9]+/', '_', strtolower($select_name));
		$id = trim($id, '_');
		echo "\n<select name='".gui_class::escape_html($select_tmp)."[]' id='$id' multiple=\"multiple\" size='$size' class='multiple $id'>\n";
		echo '<optgroup label="&nbsp;Available">';
		foreach($tokenvalues as $k => $v) {
			if($k === $noprint) continue;
			if(is_array($dflt) && (in_array($k, $dflt) )) {
				continue;
			}
			else {
				echo "  <option title=\"".gui_class::escape_html($v)."\" value='".gui_class::escape_html($k)."'>$v</option>\n";
			}
		}
		echo '</optgroup>';
		echo "</select>\n";
		echo "</td>\n";

		// Buttons
		echo "<td style=\"border: 0px\">\n" .
		"<input type=\"button\" tabindex=\"-1\" value=\">\"\n " .
		" onclick=\"moveSelectedOption(this.form.elements['".gui_class::escape_html($select_tmp)."[]'], this.form.elements['".gui_class::escape_html($select_name)."[]']);\"  class=\"button arrow-right\" style=\"margin: 7px 7px\" /><br />\n ".
		"<input type=\"button\" tabindex=\"-1\" value=\"<\" " .
		" onclick=\"moveSelectedOption(this.form.elements['".gui_class::escape_html($select_name)."[]'], this.form.elements['".gui_class::escape_html($select_tmp)."[]']);\" class=\"button arrow-left\" style=\"margin: 0px 7px\" />\n";
		echo "</td>\n";

		// Right box = selected options that will be submitted
		echo "<td style=\"border: 0px\">\n";
		echo "\n<select name='".gui_class::escape_html($select_name)."[]' id='".gui_class::escape_html($select_name)."[]' multiple=\"multiple\" size=\"$size\" class=\"multiple $id\">\n";
		echo '<optgroup label="&nbsp;Selected">';
		echo "<pre>";
		foreach($tokenvalues as $k => $v) {
			if($k === $noprint) continue;
			if(is_array($dflt) && (in_array($k, $dflt) )) {
				echo "  <option title='".gui_class::escape_html($v)."' value='".gui_class::escape_html($k)."'>".gui_class::escape_html($v)."</option>\n";
			}
		}
		echo '</optgroup>';
		echo "</select>\n";
		echo "</td>\n</tr>\n</table>\n";
		if (sizeof($tokenvalues)>$size){
			self::addFilterlist($select_tmp.'[]', sizeof($tokenvalues), $select_name.'[]');
		}
		return true;
	}

	function render($obj, $token, $value) {
		$sel_tmp = false;
		if(is_array($obj->properties['special'][$token])) {
			$sel_tmp = $obj->properties['special'][$token];
		}
		else {
			$objs = get_objects_for_user($obj->properties['special'][$token]);
			if(isset($objs)) {
				$sel_tmp = $objs;
			}
		}
		if($obj->properties['special'][$token] === $obj->type) {
			// self referencing objects get left out in their own
			// selection list
			self::make_multi_selection($sel_tmp, $value, $this->get_id($obj, $token), $obj->id);
		} else {
			self::make_multi_selection($sel_tmp, $value, $this->get_id($obj, $token));
		}
	}

	public function get_label($obj, $token, $required = false) {
		$token_label = isset($obj->properties['label'][$token]) ? $obj->properties['label'][$token] : $token;
		$extra = null;
		if($required) {
			$extra = " class='required' ";
		}

		return "<label $extra for=\"".gui_class::escape_html($this->get_id($obj, $token))."[]\">$token_label</label>";
	}

	static function validate($obj, $token, $value) {
		$spec = $obj->properties['special'];
		if (is_array($spec[$token])) {
			foreach ($value as $k => $v) {
				if (!isset($spec[$token][$v]))
					unset($value[$k]);
			}
			return $value;
		}
		// templates can link to their specific grouptypes,
		// so we need to check for that too
		foreach ($value as $id) {
			if (!$obj->object_exists($spec[$token], $id) &&
			    !$obj->object_exists($spec[$token] . '_template', $id))
			{
				throw new widget_validation_exception("Links to {$spec[$token]} type objects, but one or more of the object(s) linked to doesn't exist ($id)");
			}
		}
		return $value;
	}
}

